@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -168,8 +168,8 @@ static ssize_t get_string_from_ident_list(prelude_string_t **out, uint64_t *iden
 static ssize_t get_string_from_result_ident(prelude_string_t **out, preludedb_result_idents_t *res)
 {
         int ret;
-        size_t i = 0;
         uint64_t ident;
+        unsigned int i = 0;
         prelude_bool_t need_sep = FALSE;
 
         ret = prelude_string_new(out);
@@ -180,7 +180,7 @@ static ssize_t get_string_from_result_ident(prelude_string_t **out, preludedb_re
         if ( ret < 0 )
                 goto err;
 
-        while ( preludedb_result_idents_get_next(res, &ident) ) {
+        while ( preludedb_result_idents_get(res, i, &ident) ) {
 
                 ret = prelude_string_sprintf(*out, "%s%" PRELUDE_PRIu64, (need_sep) ? ", " : "", ident);
                 if ( ret < 0 )
@@ -208,29 +208,29 @@ static ssize_t get_string_from_result_ident(prelude_string_t **out, preludedb_re
 
 
 
-int classic_delete_alert(preludedb_sql_t *sql, uint64_t ident)
+int classic_delete_alert(preludedb_t *db, uint64_t ident)
 {
         char buf[32];
 
         snprintf(buf, sizeof(buf), "= %" PRELUDE_PRIu64, ident);
 
-        return do_delete_alert(sql, buf);
+        return do_delete_alert(preludedb_get_sql(db), buf);
 }
 
 
 
-int classic_delete_heartbeat(preludedb_sql_t *sql, uint64_t ident)
+int classic_delete_heartbeat(preludedb_t *db, uint64_t ident)
 {
         char buf[32];
 
         snprintf(buf, sizeof(buf), "= %" PRELUDE_PRIu64, ident);
 
-        return do_delete_heartbeat(sql, buf);
+        return do_delete_heartbeat(preludedb_get_sql(db), buf);
 }
 
 
 
-ssize_t classic_delete_alert_from_result_idents(preludedb_sql_t *sql, preludedb_result_idents_t *results)
+ssize_t classic_delete_alert_from_result_idents(preludedb_t *db, preludedb_result_idents_t *results)
 {
         int ret;
         ssize_t count;
@@ -240,14 +240,14 @@ ssize_t classic_delete_alert_from_result_idents(preludedb_sql_t *sql, preludedb_
         if ( count <= 0 )
                 return count;
 
-        ret = do_delete_alert(sql, prelude_string_get_string(buf));
+        ret = do_delete_alert(preludedb_get_sql(db), prelude_string_get_string(buf));
         prelude_string_destroy(buf);
 
         return (ret < 0) ? ret : count;
 }
 
 
-ssize_t classic_delete_alert_from_list(preludedb_sql_t *sql, uint64_t *ident, size_t size)
+ssize_t classic_delete_alert_from_list(preludedb_t *db, uint64_t *ident, size_t size)
 {
         int ret;
         ssize_t count;
@@ -257,7 +257,7 @@ ssize_t classic_delete_alert_from_list(preludedb_sql_t *sql, uint64_t *ident, si
         if ( count < 0 )
                 return count;
 
-        ret = do_delete_alert(sql, prelude_string_get_string(buf));
+        ret = do_delete_alert(preludedb_get_sql(db), prelude_string_get_string(buf));
         prelude_string_destroy(buf);
 
         return (ret < 0) ? ret : count;
@@ -265,7 +265,7 @@ ssize_t classic_delete_alert_from_list(preludedb_sql_t *sql, uint64_t *ident, si
 
 
 
-ssize_t classic_delete_heartbeat_from_result_idents(preludedb_sql_t *sql, preludedb_result_idents_t *results)
+ssize_t classic_delete_heartbeat_from_result_idents(preludedb_t *db, preludedb_result_idents_t *results)
 {
         int ret;
         ssize_t count;
@@ -275,7 +275,7 @@ ssize_t classic_delete_heartbeat_from_result_idents(preludedb_sql_t *sql, prelud
         if ( count <= 0 )
                 return count;
 
-        ret = do_delete_heartbeat(sql, prelude_string_get_string(buf));
+        ret = do_delete_heartbeat(preludedb_get_sql(db), prelude_string_get_string(buf));
         prelude_string_destroy(buf);
 
         return (ret < 0) ? ret : count;
@@ -283,7 +283,7 @@ ssize_t classic_delete_heartbeat_from_result_idents(preludedb_sql_t *sql, prelud
 
 
 
-ssize_t classic_delete_heartbeat_from_list(preludedb_sql_t *sql, uint64_t *ident, size_t size)
+ssize_t classic_delete_heartbeat_from_list(preludedb_t *db, uint64_t *ident, size_t size)
 {
         int ret;
         ssize_t count;
@@ -293,7 +293,7 @@ ssize_t classic_delete_heartbeat_from_list(preludedb_sql_t *sql, uint64_t *ident
         if ( count < 0 )
                 return count;
 
-        ret = do_delete_heartbeat(sql, prelude_string_get_string(buf));
+        ret = do_delete_heartbeat(preludedb_get_sql(db), prelude_string_get_string(buf));
         prelude_string_destroy(buf);
 
         return (ret < 0) ? ret : count;
