@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -33,10 +33,7 @@
 #include <libprelude/idmef.h>
 #include <libprelude/idmef-tree-wrap.h>
 
-#include "preludedb-sql-settings.h"
-#include "preludedb-sql.h"
-#include "preludedb-error.h"
-
+#include "preludedb.h"
 #include "classic-get.h"
 
 #define db_log(sql) prelude_log(PRELUDE_LOG_ERR, "%s\n", prelude_sql_error(sql))
@@ -51,7 +48,7 @@ static int _get_ ## name (preludedb_sql_t *sql, preludedb_sql_row_t *row,
         type *value;                                                                            \
         int ret;                                                                                \
                                                                                                 \
-        ret = preludedb_sql_row_fetch_field(row, index, &field);                                \
+        ret = preludedb_sql_row_get_field(row, index, &field);                                \
         if ( ret <= 0 )                                                                         \
                 return ret;                                                                     \
                                                                                                 \
@@ -80,6 +77,8 @@ get_(float, float)
         _get_float(sql, row, index, parent, (int (*)(void *, float **)) parent_new_child)
 
 
+int classic_unescape_binary_safe(preludedb_sql_t *sql, preludedb_sql_field_t *field,
+                                 idmef_additional_data_type_t type, unsigned char **output, size_t *outsize);
 
 
 static int _get_string(preludedb_sql_t *sql, preludedb_sql_row_t *row,
@@ -90,7 +89,7 @@ static int _get_string(preludedb_sql_t *sql, preludedb_sql_row_t *row,
         prelude_string_t *string;
         int ret;
 
-        ret = preludedb_sql_row_fetch_field(row, index, &field);
+        ret = preludedb_sql_row_get_field(row, index, &field);
         if ( ret <= 0 )
                 return ret;
 
@@ -114,7 +113,7 @@ static int _get_string_listed(preludedb_sql_t *sql, preludedb_sql_row_t *row,
         prelude_string_t *string;
         int ret;
 
-        ret = preludedb_sql_row_fetch_field(row, index, &field);
+        ret = preludedb_sql_row_get_field(row, index, &field);
         if ( ret <= 0 )
                 return ret;
 
@@ -139,7 +138,7 @@ static int _get_enum(preludedb_sql_t *sql, preludedb_sql_row_t *row,
         int *enum_val;
         int ret;
 
-        ret = preludedb_sql_row_fetch_field(row, index, &field);
+        ret = preludedb_sql_row_get_field(row, index, &field);
         if ( ret <= 0 )
                 return ret;
 
@@ -165,16 +164,16 @@ static int _get_timestamp(preludedb_sql_t *sql, preludedb_sql_row_t *row,
         idmef_time_t *time;
         int ret;
 
-        ret = preludedb_sql_row_fetch_field(row, time_index, &time_field);
+        ret = preludedb_sql_row_get_field(row, time_index, &time_field);
         if ( ret <= 0 )
                 return ret;
 
-        ret = preludedb_sql_row_fetch_field(row, gmtoff_index, &gmtoff_field);
+        ret = preludedb_sql_row_get_field(row, gmtoff_index, &gmtoff_field);
         if ( ret <= 0 )
                 return (ret < 0) ? ret : -1;
 
         if ( usec_index != -1 ) {
-                ret = preludedb_sql_row_fetch_field(row, usec_index, &usec_field);
+                ret = preludedb_sql_row_get_field(row, usec_index, &usec_field);
                 if ( ret <= 0 )
                         return (ret < 0) ? ret : -1;
 
@@ -1157,7 +1156,7 @@ static int get_file_access(preludedb_sql_t *sql,
         if ( ret <= 0 )
                 goto error;
 
-        ret = preludedb_sql_row_fetch_field(row, 0, &field);
+        ret = preludedb_sql_row_get_field(row, 0, &field);
         if ( ret <= 0 )
                 goto error;
 
@@ -1581,18 +1580,23 @@ static int get_target(preludedb_sql_t *sql,
         return ret;
 }
 
+
 static int get_additional_data(preludedb_sql_t *sql,
                                uint64_t message_ident,
                                char parent_type,
                                void *parent,
                                int (*parent_new_child)(void *, idmef_additional_data_t **, int pos))
 {
+        int ret = 0;
+        char *svalue = NULL;
+        size_t svalue_size;
+        prelude_bool_t svalue_need_free;
         preludedb_sql_table_t *table;
         preludedb_sql_row_t *row;
+        idmef_additional_data_type_t type;
         idmef_additional_data_t *additional_data;
         idmef_data_t *data;
         preludedb_sql_field_t *field;
-        int ret = 0;
 
         ret = preludedb_sql_query_sprintf(sql, &table,
                                           "SELECT type, meaning, data "
@@ -1618,7 +1622,7 @@ static int get_additional_data(preludedb_sql_t *sql,
                 if ( ret < 0 )
                         goto error;
 
-                ret = preludedb_sql_row_fetch_field(row, 2, &field);
+                ret = preludedb_sql_row_get_field(row, 2, &field);
                 if ( ret <= 0 )
                         goto error;
 
@@ -1626,87 +1630,85 @@ static int get_additional_data(preludedb_sql_t *sql,
                 if ( ret < 0 )
                         goto error;
 
-                switch ( idmef_additional_data_get_type(additional_data) ) {
-                case IDMEF_ADDITIONAL_DATA_TYPE_BOOLEAN: case IDMEF_ADDITIONAL_DATA_TYPE_BYTE: {
-                        uint8_t value;
-
-                        ret = preludedb_sql_field_to_uint8(field, &value);
-                        if ( ret < 0 )
-                                ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        else
-                                idmef_data_set_byte(data, value);
+                type = idmef_additional_data_get_type(additional_data);
 
+                ret = classic_unescape_binary_safe(sql, field, type, (unsigned char **) &svalue, &svalue_size);
+                if ( ret < 0 )
                         break;
-                }
 
+                svalue_need_free = TRUE;
+                switch ( type ) {
                 case IDMEF_ADDITIONAL_DATA_TYPE_CHARACTER: {
-                        uint8_t value;
-
-                        ret = preludedb_sql_field_to_uint8(field, &value);
-                        if ( ret < 0 )
-                                ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        else
-                                idmef_data_set_char(data, (char) value);
-
+                        idmef_data_set_char(data, (char) *svalue);
                         break;
                 }
 
-                case IDMEF_ADDITIONAL_DATA_TYPE_DATE_TIME:
-                case IDMEF_ADDITIONAL_DATA_TYPE_PORTLIST:
-                case IDMEF_ADDITIONAL_DATA_TYPE_STRING:
-                case IDMEF_ADDITIONAL_DATA_TYPE_XML:
-                        ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        break;
-
                 case IDMEF_ADDITIONAL_DATA_TYPE_REAL: {
                         float value;
 
-                        ret = preludedb_sql_field_to_float(field, &value);
-                        if ( ret < 0 )
-                                ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        else
-                                idmef_data_set_float(data, value);
+                        ret = sscanf(svalue, "%f", &value);
+                        if ( ret <= 0 )
+                                break;
 
+                        idmef_data_set_float(data, value);
+                        break;
+                }
+
+                case IDMEF_ADDITIONAL_DATA_TYPE_BYTE:
+                case IDMEF_ADDITIONAL_DATA_TYPE_BOOLEAN: {
+                        uint8_t value;
+
+                        ret = sscanf(svalue, "%" PRELUDE_SCNu8, &value);
+                        if ( ret <= 0 )
+                                break;
+
+                        idmef_data_set_byte(data, value);
                         break;
                 }
 
                 case IDMEF_ADDITIONAL_DATA_TYPE_INTEGER: {
-                        uint32_t value;
+                        int64_t value;
 
-                        ret = preludedb_sql_field_to_uint32(field, &value);
-                        if ( ret < 0 )
-                                ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        else
-                                idmef_data_set_uint32(data, value);
-
-                        break;
-                }
-
-                case IDMEF_ADDITIONAL_DATA_TYPE_BYTE_STRING: {
-                        unsigned char *value;
-                        size_t value_size;
-
-                        ret = preludedb_sql_unescape_binary(sql,
-                                                            preludedb_sql_field_get_value(field),
-                                                            preludedb_sql_field_get_len(field),
-                                                            &value, &value_size);
-                        if ( ret < 0 )
+                        ret = sscanf(svalue, "%" PRELUDE_SCNd64, &value);
+                        if ( ret <= 0 )
                                 break;
 
-                        ret = idmef_data_set_byte_string_nodup(data, value, value_size);
-
+                        idmef_data_set_int(data, value);
                         break;
                 }
 
                 case IDMEF_ADDITIONAL_DATA_TYPE_NTPSTAMP: {
                         uint64_t value;
 
-                        ret = preludedb_sql_field_to_uint64(field, &value);
-                        if ( ret < 0 )
-                                ret = idmef_data_set_char_string_dup(data, preludedb_sql_field_get_value(field));
-                        else
-                                idmef_data_set_uint64(data, value);
+                        ret = sscanf(svalue, "%" PRELUDE_SCNu64, &value);
+                        if ( ret <= 0 )
+                                break;
 
+                        idmef_data_set_int(data, value);
+                        break;
+                }
+
+                case IDMEF_ADDITIONAL_DATA_TYPE_BYTE_STRING:
+                        svalue_need_free = FALSE;
+                        ret = idmef_data_set_byte_string_nodup(data, (unsigned char *) svalue, svalue_size);
+                        break;
+
+                case IDMEF_ADDITIONAL_DATA_TYPE_PORTLIST:
+                case IDMEF_ADDITIONAL_DATA_TYPE_STRING:
+                case IDMEF_ADDITIONAL_DATA_TYPE_XML: {
+                        svalue_need_free = FALSE;
+                        ret = idmef_data_set_char_string_nodup_fast(data, (char *) svalue, svalue_size);
+                        break;
+                }
+
+                case IDMEF_ADDITIONAL_DATA_TYPE_DATE_TIME: {
+                        idmef_time_t *time;
+
+                        ret = idmef_time_new_from_string(&time, svalue);
+                        if ( ret < 0 )
+                                return ret;
+
+                        idmef_data_set_time(data, time);
                         break;
                 }
 
@@ -1714,9 +1716,11 @@ static int get_additional_data(preludedb_sql_t *sql,
                         ret = -1;
                 }
 
+                if ( svalue_need_free )
+                        free(svalue);
+
                 if ( ret < 0 )
                         goto error;
-
         }
 
  error:
@@ -1975,7 +1979,7 @@ static int get_overflow_alert(preludedb_sql_t *sql,
         if ( ret < 0 )
                 goto error;
 
-        ret = preludedb_sql_row_fetch_field(row, 2, &field);
+        ret = preludedb_sql_row_get_field(row, 2, &field);
         if ( ret < 0 )
                 goto error;
 
@@ -2026,8 +2030,9 @@ static int get_alert_messageid(preludedb_sql_t *sql, uint64_t ident, idmef_alert
 }
 
 
-int classic_get_alert(preludedb_sql_t *sql, uint64_t ident, idmef_message_t **message)
+int classic_get_alert(preludedb_t *db, uint64_t ident, idmef_message_t **message)
 {
+        preludedb_sql_t *sql = preludedb_get_sql(db);
         idmef_alert_t *alert;
         int ret;
 
@@ -2133,8 +2138,9 @@ static int _get_heartbeat(preludedb_sql_t *sql, uint64_t ident, idmef_heartbeat_
 
 
 
-int classic_get_heartbeat(preludedb_sql_t *sql, uint64_t ident, idmef_message_t **message)
+int classic_get_heartbeat(preludedb_t *db, uint64_t ident, idmef_message_t **message)
 {
+        preludedb_sql_t *sql = preludedb_get_sql(db);
         idmef_heartbeat_t *heartbeat;
         int ret;
 
