@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -35,14 +35,13 @@
 #include "preludedb-sql.h"
 
 #include "classic-sql-join.h"
-#include "classic-sql-select.h"
 #include "classic-path-resolve.h"
 
 #define FIELD_CONTEXT_WHERE    1
 #define FIELD_CONTEXT_SELECT   2
 #define FIELD_CONTEXT_FUNCTION 3
 
-
+int classic_get_path_column_count(preludedb_selected_path_t *selected);
 
 static int default_table_name_resolver(const idmef_path_t *path, char **table_name)
 {
@@ -97,7 +96,6 @@ static int default_field_name_resolver(const idmef_path_t *path, int field_conte
 }
 
 
-
 static int time_with_usec_field_name_resolver(const idmef_path_t *path, int field_context,
                                               const char *table_name, prelude_string_t *output)
 {
@@ -115,7 +113,7 @@ static int time_without_usec_field_name_resolver(const idmef_path_t *path, int f
         const char *child_name = idmef_path_get_name(path, idmef_path_get_depth(path) - 1);
 
         if ( field_context == FIELD_CONTEXT_SELECT )
-                return prelude_string_sprintf(output, "%s.%s, %s.%s_gmtoff, 0",
+                return prelude_string_sprintf(output, "%s.%s, %s.%s_gmtoff",
                                               table_name, child_name, table_name, child_name);
 
         return prelude_string_sprintf(output, "%s.%s", table_name, child_name);
@@ -240,6 +238,21 @@ static int file_field_name_resolver(const idmef_path_t *path, int field_context,
 }
 
 
+
+static int addata_field_name_resolver(const idmef_path_t *path, int field_context, const char *table_name,
+                                    prelude_string_t *output)
+{
+        const char *child_name = idmef_path_get_name(path, idmef_path_get_depth(path) - 1);
+
+        if (  field_context == FIELD_CONTEXT_SELECT && strcmp(child_name, "data") == 0 )
+                return prelude_string_sprintf(output, "%s.%s, %s.type",
+                                              table_name, child_name, table_name);
+
+        return prelude_string_sprintf(output, "%s.%s", table_name, child_name);
+}
+
+
+
 typedef struct classic_idmef_class {
         idmef_class_id_t id;
         int (*resolve_table_name)(const idmef_path_t *path, char **table_name);
@@ -257,6 +270,7 @@ static const classic_idmef_class_t classes[] = {
         { IDMEF_CLASS_ID_CHECKSUM, default_table_name_resolver, checksum_field_name_resolver },
         { IDMEF_CLASS_ID_FILE, default_table_name_resolver, file_field_name_resolver },
         { IDMEF_CLASS_ID_FILE_ACCESS, file_access_table_name_resolver, default_field_name_resolver },
+        { IDMEF_CLASS_ID_ADDITIONAL_DATA, default_table_name_resolver, addata_field_name_resolver },
 };
 
 
@@ -285,9 +299,22 @@ static const classic_idmef_class_t *search_path(const idmef_path_t *path)
 
 
 
-static int classic_path_resolve(const idmef_path_t *path, int field_context,
-                                classic_sql_join_t *join, prelude_string_t *output)
+static int get_field_context(preludedb_selected_path_t *selected)
 {
+        preludedb_selected_object_t *object;
+
+        object = preludedb_selected_path_get_object(selected);
+        if ( preludedb_selected_object_is_function(object) || preludedb_selected_path_get_flags(selected) & PRELUDEDB_SELECTED_PATH_FLAGS_GROUP_BY )
+                return FIELD_CONTEXT_FUNCTION;
+        else
+                return FIELD_CONTEXT_SELECT;
+}
+
+
+
+static int _classic_path_resolve(const idmef_path_t *path, int field_context, void *data, prelude_string_t *output)
+{
+        classic_sql_join_t *join = data;
         const classic_idmef_class_t *class;
         classic_sql_joined_table_t *table;
         char *table_name;
@@ -315,61 +342,10 @@ static int classic_path_resolve(const idmef_path_t *path, int field_context,
 }
 
 
-
-int classic_path_resolve_selected(preludedb_sql_t *sql,
-                                  preludedb_selected_path_t *selected,
-                                  classic_sql_join_t *join, classic_sql_select_t *select)
+int classic_path_resolve(preludedb_selected_path_t *selpath, preludedb_selected_object_t *object, void *data, prelude_string_t *output)
 {
-        idmef_path_t *path;
-        preludedb_selected_path_flags_t flags;
-        prelude_string_t *field_name;
-        int ret;
-        int field_context;
-
-        ret = prelude_string_new(&field_name);
-        if ( ret < 0 )
-                return ret;
-
-        path = preludedb_selected_path_get_path(selected);
-        flags = preludedb_selected_path_get_flags(selected);
-
-        if ( flags & (PRELUDEDB_SELECTED_OBJECT_FUNCTION_MIN|
-                      PRELUDEDB_SELECTED_OBJECT_FUNCTION_MAX|
-                      PRELUDEDB_SELECTED_OBJECT_FUNCTION_AVG|
-                      PRELUDEDB_SELECTED_OBJECT_FUNCTION_STD|
-                      PRELUDEDB_SELECTED_OBJECT_FUNCTION_COUNT) )
-                field_context = FIELD_CONTEXT_FUNCTION;
-        else
-                field_context = FIELD_CONTEXT_SELECT;
-
-        ret = classic_path_resolve(path, field_context, join, field_name);
-        if ( ret < 0 )
-                goto error;
-
-        ret = classic_sql_select_add_field(select, prelude_string_get_string(field_name), flags);
-
- error:
-        prelude_string_destroy(field_name);
-
-        return ret;
-}
-
-
-
-int classic_path_resolve_selection(preludedb_sql_t *sql,
-                                   preludedb_path_selection_t *selection,
-                                   classic_sql_join_t *join, classic_sql_select_t *select)
-{
-        preludedb_selected_path_t *selected = NULL;
-        int ret;
-
-        while ( (selected = preludedb_path_selection_get_next(selection, selected)) ) {
-                ret = classic_path_resolve_selected(sql, selected, join, select);
-                if ( ret < 0 )
-                        return ret;
-        }
-
-        return 0;
+        const idmef_path_t *path = preludedb_selected_object_get_data(object);
+        return _classic_path_resolve(path, get_field_context(selpath), data, output);
 }
 
 
@@ -385,7 +361,7 @@ static int classic_path_resolve_criterion(preludedb_sql_t *sql,
         if ( ret < 0 )
                 return ret;
 
-        ret = classic_path_resolve(idmef_criterion_get_path(criterion), FIELD_CONTEXT_WHERE, join, field_name);
+        ret = _classic_path_resolve(idmef_criterion_get_path(criterion), FIELD_CONTEXT_WHERE, join, field_name);
         if ( ret < 0 )
                 goto error;
 
