@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,17 +15,17 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 #ifndef _LIBPRELUDEDB_CLASSIC_GET_H
 #define _LIBPRELUDEDB_CLASSIC_GET_H
 
-int classic_get_alert(preludedb_sql_t *sql, uint64_t ident, idmef_message_t **message);
+int classic_get_alert(preludedb_t *db, uint64_t ident, idmef_message_t **message);
 
-int classic_get_heartbeat(preludedb_sql_t *sql, uint64_t ident, idmef_message_t **message);
+int classic_get_heartbeat(preludedb_t *db, uint64_t ident, idmef_message_t **message);
 
 #endif /* ! _LIBPRELUDEDB_CLASSIC_GET_H  */
