@@ -16,15 +16,15 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 #ifndef _LIBPRELUDEDB_CLASSIC_INSERT_H
 #define _LIBPRELUDEDB_CLASSIC_INSERT_H
 
-int classic_insert(preludedb_sql_t *sql, idmef_message_t *message);
+int classic_insert(preludedb_t *db, idmef_message_t *message);
 
 #endif /* _LIBPRELUDEDB_CLASSIC_INSERT_H */
