@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -25,12 +25,8 @@
 #define _LIBPRELUDEDB_CLASSIC_PATH_RESOLVE_H
 
 
-int classic_path_resolve_selected(preludedb_sql_t *sql,
-				  preludedb_selected_path_t *selected,
-				  classic_sql_join_t *join, classic_sql_select_t *select);
-int classic_path_resolve_selection(preludedb_sql_t *sql,
-				   preludedb_path_selection_t *selection,
-				   classic_sql_join_t *join, classic_sql_select_t *select);
+int classic_path_resolve(preludedb_selected_path_t *selpath, preludedb_selected_object_t *object, void *data, prelude_string_t *output);
+
 int classic_path_resolve_criteria(preludedb_sql_t *sql,
 				  idmef_criteria_t *criteria,
 				  classic_sql_join_t *join, prelude_string_t *output);
