@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,15 +15,16 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 #include "config.h"
 
 #include <stdio.h>
+#include <stddef.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -36,6 +37,7 @@
 #include <errmsg.h>
 
 #include <libprelude/idmef.h>
+#include <libprelude/common.h>
 #include <libprelude/prelude-error.h>
 #include <libprelude/prelude-string.h>
 
@@ -55,6 +57,20 @@
 #endif /* ! MYSQL_VERSION_ID */
 
 
+
+typedef struct {
+        MYSQL_ROW *row;
+        unsigned long lengths[1];
+} mysql_row_data_t;
+
+
+typedef struct {
+        MYSQL session;
+        int support_timezone;
+} mysql_session_t;
+
+
+
 int mysql_LTX_prelude_plugin_version(void);
 int mysql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data);
 
@@ -95,24 +111,56 @@ static int handle_error(void *session, prelude_error_code_t code)
 
 
 
+static int check_timezone_support(void *session)
+{
+        int ret;
+        MYSQL_ROW row;
+        MYSQL_RES *results;
+
+        ret = mysql_query(session, "SELECT CONVERT_TZ(NOW(), 'SYSTEM', 'UTC');");
+        if ( ret != 0 )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "timezone table check query error");
+
+        results = mysql_store_result(session);
+        if ( ! results )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "timezone table check returned no results");
+
+        row = mysql_fetch_row(results);
+        if ( ! row ) {
+                mysql_free_result(results);
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "timezone table check returned no rows");
+        }
+
+        ((mysql_session_t *) session)->support_timezone = (row[0] != NULL);
+        mysql_free_result(results);
+
+        return 0;
+}
+
+
 static int sql_open(preludedb_sql_settings_t *settings, void **session)
 {
         int ret;
         unsigned int port = 0;
 
-        if ( preludedb_sql_settings_get_port(settings) )
-                port = atoi(preludedb_sql_settings_get_port(settings));
-
-        *session = mysql_init(NULL);
+        *session = malloc(sizeof(mysql_session_t));
         if ( ! *session )
                 return preludedb_error_from_errno(errno);
 
+        if ( ! mysql_init(*session) ) {
+                free(*session);
+                return preludedb_error_from_errno(errno);
+        }
+
+        if ( preludedb_sql_settings_get_port(settings) )
+                port = atoi(preludedb_sql_settings_get_port(settings));
+
         if ( ! mysql_real_connect(*session,
                                   preludedb_sql_settings_get_host(settings),
                                   preludedb_sql_settings_get_user(settings),
                                   preludedb_sql_settings_get_pass(settings),
                                   preludedb_sql_settings_get_name(settings),
-                                  port, NULL, 0) ) {
+                                  port, NULL, CLIENT_MULTI_STATEMENTS) ) {
 
                 ret = handle_error(*session, PRELUDEDB_ERROR_CONNECTION);
                 mysql_close(*session);
@@ -122,7 +170,7 @@ static int sql_open(preludedb_sql_settings_t *settings, void **session)
 
         mysql_query(*session, "SET SESSION wait_timeout=" WAIT_TIMEOUT_VALUE);
 
-        return 0;
+        return check_timezone_support(*session);
 }
 
 
@@ -186,30 +234,69 @@ static int sql_build_limit_offset_string(void *session, int limit, int offset, p
 
 
 
-static int sql_query(void *session, const char *query, void **resource)
+static int sql_query(void *session, const char *query, preludedb_sql_table_t **table)
 {
-        if ( mysql_query(session, query) != 0 )
+        int ret;
+        MYSQL_RES *result;
+
+        ret = mysql_query(session, query);
+        if ( ret != 0 )
                 return handle_error(session, PRELUDEDB_ERROR_QUERY);
 
-        *resource = mysql_store_result(session);
-        if ( *resource ) {
-                if ( mysql_num_rows(*resource) == 0 ) {
-                        mysql_free_result(*resource);
-                        return 0;
-                }
+        do {
+                result = mysql_store_result(session);
+                if ( ! result && mysql_field_count(session) > 0 )
+                        return handle_error(session, PRELUDEDB_ERROR_QUERY);
 
-                return 1;
+                /*
+                 * If there are multiple results, process only the last one.
+                 * 0 -> more results, -1 no more results, > 0 error.
+                 */
+                ret = mysql_next_result(session);
+                if ( ret != -1 ) {
+                        if ( result )
+                                mysql_free_result(result);
+
+                        if ( ret > 0 )
+                                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "mysql_next_result() error: %s", mysql_error(session));
+                }
+        } while ( ret == 0 );
+
+        /*
+         * Last result
+         */
+        if ( ! result )
+                return 0;
+
+        if ( mysql_num_rows(result) == 0 ) {
+                mysql_free_result(result);
+                return 0;
         }
 
-        return mysql_errno(session) ? handle_error(session, PRELUDEDB_ERROR_QUERY) : 0;
+        ret = preludedb_sql_table_new(table, result);
+        if ( ret < 0 ) {
+                mysql_free_result(result);
+                return ret;
+        }
+
+        return 1;
 }
 
 
 
-static void sql_resource_destroy(void *session, void *resource)
+static int sql_get_last_insert_ident(void *session, uint64_t *ident)
 {
-        if ( resource )
-                mysql_free_result(resource);
+        *ident = mysql_insert_id(session);
+        if ( *ident == 0 )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "could not retrieve last insert ID");
+
+        return 0;
+}
+
+
+static void sql_table_destroy(void *session, preludedb_sql_table_t *table)
+{
+        mysql_free_result(preludedb_sql_table_get_data(table));
 }
 
 
@@ -221,83 +308,119 @@ static MYSQL_FIELD *get_field(MYSQL_RES *res, unsigned int column_num)
 
 
 
-static const char *sql_get_column_name(void *session, void *resource, unsigned int column_num)
+static const char *sql_get_column_name(void *session, preludedb_sql_table_t *table, unsigned int column_num)
 {
         MYSQL_FIELD *field;
 
-        field = get_field(resource, column_num);
+        field = get_field(preludedb_sql_table_get_data(table), column_num);
 
         return field ? field->name : NULL;
 }
 
 
 
-static int sql_get_column_num(void *session, void *resource, const char *column_name)
+static int sql_get_column_num(void *session, preludedb_sql_table_t *table, const char *column_name)
 {
+        int fields_num, i;
         MYSQL_FIELD *fields;
-        int fields_num;
-        int i;
+        MYSQL_RES *result = preludedb_sql_table_get_data(table);
 
-        fields = mysql_fetch_fields(resource);
+        fields = mysql_fetch_fields(result);
         if ( ! fields )
-                return -1;
-
-        fields_num = mysql_num_fields(resource);
+                return prelude_error_verbose(PRELUDEDB_ERROR_GENERIC, "mysql_fetch_fields() error while looking up column '%s'", column_name);
 
+        fields_num = mysql_num_fields(result);
         for ( i = 0; i < fields_num; i++ ) {
                 if ( strcmp(column_name, fields[i].name) == 0 )
                         return i;
         }
 
-        return -1;
+        return prelude_error_verbose(PRELUDEDB_ERROR_GENERIC, "unknown column '%s'", column_name);
 }
 
 
 
-static unsigned int sql_get_column_count(void *session, void *resource)
+static unsigned int sql_get_column_count(void *session, preludedb_sql_table_t *table)
 {
-        return mysql_num_fields(resource);
+        return mysql_num_fields(preludedb_sql_table_get_data(table));
 }
 
 
 
-static unsigned int sql_get_row_count(void *session, void *resource)
+static unsigned int sql_get_row_count(void *session, preludedb_sql_table_t *table)
 {
-        return (unsigned int) mysql_num_rows(resource);
+        return (unsigned int) mysql_num_rows(preludedb_sql_table_get_data(table));
 }
 
 
-
-static int sql_fetch_row(void *session, void *resource, void **row)
+static void sql_destroy_row(void *session, preludedb_sql_table_t *table, preludedb_sql_row_t *row)
 {
-        *row = mysql_fetch_row(resource);
-        if ( ! *row )
-                return mysql_errno(session) ? preludedb_error(PRELUDEDB_ERROR_GENERIC) : 0;
+        free(preludedb_sql_row_get_data(row));
+}
+
+
+static int sql_fetch_row(void *session, preludedb_sql_table_t *table, unsigned int row_index, preludedb_sql_row_t **rrow)
+{
+        int ret;
+        void *row;
+        mysql_row_data_t *myrow;
+        unsigned long *lengths;
+        unsigned int column_count, i;
+        MYSQL_RES *result = preludedb_sql_table_get_data(table);
+
+        column_count = preludedb_sql_table_get_column_count(table);
+
+        while ( preludedb_sql_table_get_fetched_row_count(table) <= row_index ) {
+                row = mysql_fetch_row(result);
+                if ( ! row ) {
+                        ret = mysql_errno(session);
+                        if ( ret )
+                                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, mysql_error(session));
+
+                        return 0;
+                }
+
+                lengths = mysql_fetch_lengths(result);
+                if ( ! lengths )
+                        return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+
+                ret = preludedb_sql_table_new_row(table, rrow, preludedb_sql_table_get_fetched_row_count(table));
+                if ( ret < 0 )
+                        return ret;
+
+                myrow = malloc(offsetof(mysql_row_data_t, lengths) + column_count * (sizeof(unsigned long)));
+                if ( ! myrow ) {
+                        preludedb_sql_row_destroy(*rrow);
+                        return preludedb_error_from_errno(errno);
+                }
+
+                for ( i = 0; i < column_count; i++ )
+                        myrow->lengths[i] = lengths[i];
+
+                myrow->row = row;
+                preludedb_sql_row_set_data(*rrow, myrow);
+        }
 
         return 1;
 }
 
 
 
-static int sql_fetch_field(void *session, void *resource, void *row,
-                           unsigned int column_num, const char **value, size_t *len)
+static int sql_fetch_field(void *session, preludedb_sql_table_t *table, preludedb_sql_row_t *row,
+                           unsigned int column_num, preludedb_sql_field_t **field)
 {
-        unsigned long *lengths;
+        mysql_row_data_t *d = preludedb_sql_row_get_data(row);
+        void *data;
+        size_t dlen = 0;
 
-        if ( column_num >= mysql_num_fields(resource) )
+        if ( column_num >= mysql_num_fields(preludedb_sql_table_get_data(table)) )
                 return preludedb_error(PRELUDEDB_ERROR_INVALID_COLUMN_NUM);
 
-        lengths = mysql_fetch_lengths(resource);
-        if ( ! lengths )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+        data = d->row[column_num];
+        if ( data )
+                dlen = d->lengths[column_num];
 
-        if ( ! ((MYSQL_ROW) row)[column_num] )
-                return 0;
-
-        *value = ((MYSQL_ROW) row)[column_num];
-        *len = lengths[column_num];
-
-        return 1;
+        return preludedb_sql_row_new_field(row, field, column_num, data, dlen);
 }
 
 
@@ -344,7 +467,7 @@ static const char *get_operator_string(idmef_criterion_operator_t operator)
 
 
 
-static int sql_build_constraint_string(prelude_string_t *out, const char *field,
+static int sql_build_constraint_string(void *session, prelude_string_t *out, const char *field,
                                        idmef_criterion_operator_t operator, const char *value)
 {
         const char *op_str;
@@ -361,101 +484,136 @@ static int sql_build_constraint_string(prelude_string_t *out, const char *field,
 
 
 
-static int sql_build_time_constraint_string(prelude_string_t *output, const char *field,
-                                            preludedb_sql_time_constraint_type_t type,
-                                            idmef_criterion_operator_t operator, int value, int gmt_offset)
+static int sql_build_time_extract_string(void *session, prelude_string_t *output, const char *field, preludedb_sql_time_constraint_type_t type, int gmt_offset)
 {
-        char buf[128];
-        const char *sql_operator;
         int ret;
+        char buf[128];
+
+        if ( ! gmt_offset )
+                ret = snprintf(buf, sizeof(buf), "%s", field);
+        else
+                ret = snprintf(buf, sizeof(buf), "DATE_ADD(%s, INTERVAL %d HOUR)", field, gmt_offset / 3600);
 
-        ret = snprintf(buf, sizeof(buf), "DATE_ADD(%s, INTERVAL %d HOUR)", field, gmt_offset / 3600);
         if ( ret < 0 || (size_t) ret >= sizeof(buf) )
                 return preludedb_error(PRELUDEDB_ERROR_GENERIC);
 
-        sql_operator = get_operator_string(operator);
-        if ( ! sql_operator )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
-
         switch ( type ) {
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_QUARTER:
+                return prelude_string_sprintf(output, "QUARTER(%s)", buf);
+
         case PRELUDEDB_SQL_TIME_CONSTRAINT_YEAR:
-                return prelude_string_sprintf(output, "EXTRACT(YEAR FROM %s) %s '%d'",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(YEAR FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MONTH:
-                return  prelude_string_sprintf(output, "EXTRACT(MONTH FROM %s) %s '%d'",
-                                               buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(MONTH FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_YDAY:
-                return prelude_string_sprintf(output, "DAYOFYEAR(%s) %s '%d'",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "DAYOFYEAR(%s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MDAY:
-                return prelude_string_sprintf(output, "DAYOFMONTH(%s) %s '%d'",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "DAYOFMONTH(%s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_WDAY:
-                return prelude_string_sprintf(output, "DAYOFWEEK(%s) %s '%d'",
-                                              buf, sql_operator, value % 7 + 1);
+                return prelude_string_sprintf(output, "WEEKDAY(%s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_HOUR:
-                return prelude_string_sprintf(output, "EXTRACT(HOUR FROM %s) %s '%d'",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(HOUR FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MIN:
-                return prelude_string_sprintf(output, "EXTRACT(MINUTE FROM %s) %s '%d'",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(MINUTE FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_SEC:
-                return prelude_string_sprintf(output, "EXTRACT(SECOND FROM %s) %s '%d'",
-                                              buf, sql_operator, value);
-        }
+                return prelude_string_sprintf(output, "EXTRACT(SECOND FROM %s)", buf);
 
-        /* not reached */
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_MSEC:
+                return prelude_string_sprintf(output, "(EXTRACT(MICROSECOND FROM %s) / 1000)", buf);
+
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_USEC:
+                return prelude_string_sprintf(output, "EXTRACT(MICROSECOND FROM %s)", buf);
+        }
 
         return preludedb_error(PRELUDEDB_ERROR_GENERIC);
 }
 
 
 
-static int sql_build_time_interval_string(preludedb_sql_time_constraint_type_t type, int value,
-                                          char *buf, size_t size)
+static int sql_build_time_timezone_string(void *session, prelude_string_t *output, const char *field, const char *tzvalue)
 {
-        char *type_str;
+        if ( ! ((mysql_session_t *) session)->support_timezone )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "MySQL timezones are not configured (mysql_tzinfo_to_sql /usr/share/zoneinfo | mysql -u root mysql)");
+
+        return prelude_string_sprintf(output, "CONVERT_TZ(%s, 'GMT', '%s')", field, tzvalue);
+}
+
+
+static int sql_build_time_constraint_string(void *session, prelude_string_t *output, const char *field,
+                                            preludedb_sql_time_constraint_type_t type,
+                                            idmef_criterion_operator_t operator, int value, int gmt_offset)
+{
+        const char *sql_operator;
         int ret;
 
-        switch ( type ) {
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_YEAR:
-                type_str = "YEAR";
+        ret = sql_build_time_extract_string(session, output, field, type, gmt_offset);
+        if ( ret < 0 )
+                return ret;
+
+        sql_operator = get_operator_string(operator);
+        if ( ! sql_operator )
+                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+
+        if ( type == PRELUDEDB_SQL_TIME_CONSTRAINT_WDAY )
+                value = value % 7 + 1;
+
+        return prelude_string_sprintf(output, " %s '%d'", sql_operator, value);
+}
+
+
+
+static int sql_build_time_interval_string(void *session, prelude_string_t *output, const char *field, const char *value, preludedb_selected_object_interval_t unit)
+{
+        const char *sunit;
+
+        switch ( unit ) {
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_QUARTER:
+                sunit = "QUARTER";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MONTH:
-                type_str = "MONTH";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_YEAR:
+                sunit = "YEAR";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MDAY:
-                type_str = "DAY";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_MONTH:
+                sunit = "MONTH";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_HOUR:
-                type_str = "HOUR";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_DAY:
+                sunit = "DAY";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MIN:
-                type_str = "MINUTE";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_HOUR:
+                sunit = "HOUR";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_SEC:
-                type_str = "SECOND";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_MIN:
+                sunit = "MINUTE";
+                break;
+
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_SEC:
+                sunit = "SECOND";
                 break;
 
         default:
                 return preludedb_error(PRELUDEDB_ERROR_GENERIC);
         }
 
-        ret = snprintf(buf, size, "INTERVAL %d %s", value, type_str);
+        return prelude_string_sprintf(output, "(%s + INTERVAL %s %s)", field, value, sunit);
+}
 
-        return (ret < 0 || (size_t) ret >= size) ? preludedb_error(PRELUDEDB_ERROR_GENERIC) : 0;
+
+
+static long sql_get_server_version(void *session)
+{
+        return mysql_get_server_version(session);
 }
 
 
@@ -476,18 +634,23 @@ int mysql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data)
         preludedb_plugin_sql_set_close_func(plugin, sql_close);
         preludedb_plugin_sql_set_escape_binary_func(plugin, sql_escape_binary);
         preludedb_plugin_sql_set_query_func(plugin, sql_query);
-        preludedb_plugin_sql_set_resource_destroy_func(plugin, sql_resource_destroy);
+        preludedb_plugin_sql_set_get_server_version_func(plugin, sql_get_server_version);
+        preludedb_plugin_sql_set_table_destroy_func(plugin, sql_table_destroy);
         preludedb_plugin_sql_set_get_column_count_func(plugin, sql_get_column_count);
         preludedb_plugin_sql_set_get_row_count_func(plugin, sql_get_row_count);
         preludedb_plugin_sql_set_get_column_name_func(plugin, sql_get_column_name);
         preludedb_plugin_sql_set_get_column_num_func(plugin, sql_get_column_num);
         preludedb_plugin_sql_set_get_operator_string_func(plugin, get_operator_string);
         preludedb_plugin_sql_set_fetch_row_func(plugin, sql_fetch_row);
+        preludedb_plugin_sql_set_row_destroy_func(plugin, sql_destroy_row);
         preludedb_plugin_sql_set_fetch_field_func(plugin, sql_fetch_field);
         preludedb_plugin_sql_set_build_constraint_string_func(plugin, sql_build_constraint_string);
+        preludedb_plugin_sql_set_build_time_extract_string_func(plugin, sql_build_time_extract_string);
         preludedb_plugin_sql_set_build_time_constraint_string_func(plugin, sql_build_time_constraint_string);
         preludedb_plugin_sql_set_build_time_interval_string_func(plugin, sql_build_time_interval_string);
+        preludedb_plugin_sql_set_build_time_timezone_string_func(plugin, sql_build_time_timezone_string);
         preludedb_plugin_sql_set_build_limit_offset_string_func(plugin, sql_build_limit_offset_string);
+        preludedb_plugin_sql_set_get_last_insert_ident_func(plugin, sql_get_last_insert_ident);
 
         return 0;
 }
