@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,15 +15,16 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 #include "config.h"
 
 #include <stdio.h>
+#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -47,13 +48,6 @@ int pgsql_LTX_prelude_plugin_version(void);
 int pgsql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data);
 
 
-struct pg_result {
-        PGresult *result;
-        int row;
-};
-
-
-
 static int handle_error(prelude_error_code_t code, PGconn *conn)
 {
         int ret;
@@ -86,39 +80,123 @@ static int handle_error(prelude_error_code_t code, PGconn *conn)
 }
 
 
-
-static int sql_query(void *session, const char *query, void **resource)
+static int _sql_query(void *session, const char *query, PGresult **result)
 {
-        int ret;
-        struct pg_result *res;
+        int status, ntuple;
 
-        res = calloc(1, sizeof (*res));
-        if ( ! res )
-                return prelude_error_from_errno(errno);
-
-        res->row = -1;
-
-        res->result = PQexec(session, query);
-        if ( ! res->result ) {
-                free(res);
+        *result = PQexec(session, query);
+        if ( ! *result )
                 return handle_error(PRELUDEDB_ERROR_QUERY, session);
+
+        status = PQresultStatus(*result);
+        if ( status == PGRES_TUPLES_OK ) {
+                ntuple = PQntuples(*result);
+                if ( ntuple == 0 )
+                        PQclear(*result);
+
+                return ntuple;
         }
 
-        ret = PQresultStatus(res->result);
-        if ( ret == PGRES_TUPLES_OK && PQntuples(res->result) != 0 ) {
-                *resource = res;
-                return 1;
-        }
-
-        PQclear(res->result);
-        free(res);
-        if ( ret == PGRES_TUPLES_OK || ret == PGRES_COMMAND_OK )
+        PQclear(*result);
+        if ( status == PGRES_COMMAND_OK )
                 return 0;
 
         return handle_error(PRELUDEDB_ERROR_QUERY, session);
 }
 
 
+
+static int sql_query(void *session, const char *query, preludedb_sql_table_t **table)
+{
+        int ret;
+        PGresult *result;
+
+        ret = _sql_query(session, query, &result);
+        if ( ret <= 0 )
+                return ret;
+
+        if ( ! table )
+                PQclear(result);
+        else {
+                ret = preludedb_sql_table_new(table, result);
+                if ( ret < 0 ) {
+                        PQclear(result);
+                        return ret;
+                }
+        }
+
+        return 1;
+}
+
+
+
+static int sql_get_last_insert_ident(void *session, uint64_t *ident)
+{
+        int ret;
+        char *value;
+        PGresult *result;
+
+        ret = _sql_query(session, "SELECT lastval();", &result);
+        if ( ret < 0 )
+                return ret;
+
+        else if ( ret == 0 )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_INVALID_VALUE, "sequence selection returned no data");
+
+        value = PQgetvalue(result, 0, 0);
+        PQclear(result);
+        if ( ! value )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_INVALID_VALUE, "retrieved sequence value is empty");
+
+        ret = sscanf(value, "%" PRELUDE_SCNu64, ident);
+        if ( ret <= 0 )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_INVALID_VALUE, "retrieved sequence value is invalid");
+
+        return 0;
+}
+
+
+
+static int check_settings(PGconn *session)
+{
+        int ret;
+        size_t size;
+        PGresult *result;
+        unsigned char *unescaped;
+        const char *original = "0xd3adb33f", *hex_escaped = "\\x30786433616462333366";
+
+        unescaped = PQunescapeBytea((const unsigned char *) hex_escaped, &size);
+        if ( ! unescaped )
+                return preludedb_error_from_errno(errno);
+
+        ret = -1;
+        if ( strlen(original) == size )
+                ret = memcmp(unescaped, original, size);
+
+        free(unescaped);
+
+        /*
+         * If PQunescapeBytea successfully unescaped our hex string, we are running libpq >= 9.0
+         * and no further check are required.
+         */
+        if ( ret == 0 )
+                return 0;
+
+        /*
+         * libpq < 9.0 cannot handle hexadecimal bytea output, which is the default for PostgreSQL 9.0 server.
+         * Check the setting value.
+         */
+        ret = _sql_query(session, "SELECT setting FROM pg_settings WHERE name = 'bytea_output' AND setting = 'hex';", &result);
+        if ( ret <= 0 )
+                return ret;
+
+        PQclear(result);
+        return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "PostgreSQL server >= 9.0 uses 'hex' mode for bytea output whereas libpq < 9.0 does not support it. "
+                                       "You may upgrade libpq to a newer version, or change the PostgreSQL server 'bytea_output' setting to 'escape' mode");
+}
+
+
+
 static int sql_open(preludedb_sql_settings_t *settings, void **session)
 {
         int ret;
@@ -138,8 +216,18 @@ static int sql_open(preludedb_sql_settings_t *settings, void **session)
                 return ret;
         }
 
+        ret = check_settings(conn);
+        if ( ret < 0 ) {
+                PQfinish(conn);
+                return ret;
+        }
+
         *session = conn;
 
+        ret = sql_query(conn, "SET standard_conforming_strings=on", NULL);
+        if ( ret < 0 )
+                return ret;
+
         return sql_query(conn, "SET DATESTYLE TO 'ISO'", NULL);
 }
 
@@ -153,6 +241,9 @@ static void sql_close(void *session)
 
 static int sql_escape(void *session, const char *input, size_t input_size, char **output)
 {
+#ifdef HAVE_PQESCAPESTRINGCONN
+        int error;
+#endif
         size_t rsize;
 
         rsize = input_size * 2 + 3;
@@ -165,8 +256,13 @@ static int sql_escape(void *session, const char *input, size_t input_size, char
 
         (*output)[0] = '\'';
 
+#ifdef HAVE_PQESCAPESTRINGCONN
+        rsize = PQescapeStringConn(session, (*output) + 1, input, input_size, &error);
+        if ( error )
+                return handle_error(PRELUDEDB_ERROR_GENERIC, session);
+#else
         rsize = PQescapeString((*output) + 1, input, input_size);
-
+#endif
         (*output)[rsize + 1] = '\'';
         (*output)[rsize + 2] = '\0';
 
@@ -185,19 +281,17 @@ static int sql_escape_binary(void *session, const unsigned char *input, size_t i
         if ( ret < 0 )
                 return ret;
 
+#ifdef HAVE_PQESCAPEBYTEACONN
+        ptr = PQescapeByteaConn(session, input, input_size, &dummy);
+#else
         ptr = PQescapeBytea(input, input_size, &dummy);
+#endif
         if ( ! ptr ) {
                 prelude_string_destroy(string);
                 return -1;
         }
 
-#ifdef HAVE_PQSERVERVERSION
-        if ( PQserverVersion(session) >= 80200 )
-                ret = prelude_string_sprintf(string, "E'%s'", ptr);
-        else
-#endif
-                ret = prelude_string_sprintf(string, "'%s'", ptr);
-
+        ret = prelude_string_sprintf(string, "'%s'", ptr);
         free(ptr);
 
         if ( ret < 0 ) {
@@ -241,56 +335,59 @@ static int sql_build_limit_offset_string(void *session, int limit, int offset, p
 
 
 
-static void sql_resource_destroy(void *session, void *resource)
+static void sql_table_destroy(void *session, preludedb_sql_table_t *table)
 {
-        if ( resource ) {
-                PQclear(((struct pg_result *) resource)->result);
-                free(resource);
-        }
+        PQclear(preludedb_sql_table_get_data(table));
 }
 
 
 
-static const char *sql_get_column_name(void *session, void *resource, unsigned int column_num)
+static const char *sql_get_column_name(void *session, preludedb_sql_table_t *table, unsigned int column_num)
 {
-        return PQfname(((struct pg_result *) resource)->result, column_num);
+        return PQfname(preludedb_sql_table_get_data(table), column_num);
 }
 
 
 
-static int sql_get_column_num(void *session, void *resource, const char *column_name)
+static int sql_get_column_num(void *session, preludedb_sql_table_t *table, const char *column_name)
 {
-        return PQfnumber(((struct pg_result *) resource)->result, column_name);
+        int ret;
+
+        ret = PQfnumber(preludedb_sql_table_get_data(table), column_name);
+        if ( ret < 0 )
+                return prelude_error_verbose(PRELUDEDB_ERROR_GENERIC, "unknown column '%s'", column_name);
+
+        return ret;
 }
 
 
 
-static unsigned int sql_get_column_count(void *session, void *resource)
+static unsigned int sql_get_column_count(void *session, preludedb_sql_table_t *table)
 {
-        return PQnfields(((struct pg_result *) resource)->result);
+        return PQnfields(preludedb_sql_table_get_data(table));
 }
 
 
 
-static unsigned int sql_get_row_count(void *session, void *resource)
+static unsigned int sql_get_row_count(void *session, preludedb_sql_table_t *table)
 {
-        return PQntuples(((struct pg_result *) resource)->result);
+        return PQntuples(preludedb_sql_table_get_data(table));
 }
 
 
 
-static int sql_fetch_row(void *s, void *resource, void **row)
+static int sql_fetch_row(void *s, preludedb_sql_table_t *table, unsigned int row_index, preludedb_sql_row_t **row)
 {
-        struct pg_result *res = resource;
+        int ret;
+        unsigned int row_count;
 
-        /*
-         * initialize *row, but we won't use it since we access row's fields directly
-         * through the PGresult structure
-         */
-        *row = NULL;
+        row_count = PQntuples(preludedb_sql_table_get_data(table));
+        if ( row_index < row_count ) {
+                ret = preludedb_sql_table_new_row(table, row, row_index);
+                if ( ret < 0 )
+                        return ret;
 
-        if ( res->row + 1 < PQntuples(res->result) ) {
-                res->row++;
+                preludedb_sql_row_set_data(*row, (void *) (unsigned long) row_index);
                 return 1;
         }
 
@@ -299,23 +396,30 @@ static int sql_fetch_row(void *s, void *resource, void **row)
 
 
 
-static int sql_fetch_field(void *session, void *resource, void *r,
-                           unsigned int column_num, const char **value, size_t *len)
+static int sql_fetch_field(void *session, preludedb_sql_table_t *table, preludedb_sql_row_t *row,
+                           unsigned int column_num, preludedb_sql_field_t **field)
 {
-        int nfields;
-        struct pg_result *res = resource;
+        char *value;
+        void *valaddr = preludedb_sql_row_get_data(row);
+        PGresult *result = preludedb_sql_table_get_data(table);
+        int nfields, len;
+        unsigned int row_index;
 
-        nfields = PQnfields(res->result);
+        row_index = (unsigned int) (unsigned long) valaddr;
+
+        nfields = PQnfields(result);
         if ( nfields < 0 || column_num >= (unsigned int) nfields )
                 return preludedb_error(PRELUDEDB_ERROR_INVALID_COLUMN_NUM);
 
-        if ( PQgetisnull(res->result, res->row, column_num) )
-                return 0;
+        if ( PQgetisnull(result, row_index, column_num) ) {
+                value = NULL;
+                len = 0;
+        } else {
+                value = PQgetvalue(result, row_index, column_num);
+                len = PQgetlength(result, row_index, column_num);
+        }
 
-        *value = PQgetvalue(res->result, res->row, column_num);
-        *len = PQgetlength(res->result, res->row, column_num);
-
-        return 1;
+        return preludedb_sql_row_new_field(row, field, column_num, value, len);
 }
 
 
@@ -361,7 +465,7 @@ static const char *get_operator_string(idmef_criterion_operator_t operator)
 
 
 
-static int sql_build_constraint_string(prelude_string_t *out, const char *field,
+static int sql_build_constraint_string(void *session, prelude_string_t *out, const char *field,
                                        idmef_criterion_operator_t operator, const char *value)
 {
         const char *op_str;
@@ -384,54 +488,53 @@ static int sql_build_constraint_string(prelude_string_t *out, const char *field,
 
 
 
-static int sql_build_time_constraint_string(prelude_string_t *output, const char *field,
-                                            preludedb_sql_time_constraint_type_t type,
-                                            idmef_criterion_operator_t operator, int value, int gmt_offset)
+static int sql_build_time_extract_string(void *session, prelude_string_t *output, const char *field,
+                                         preludedb_sql_time_constraint_type_t type, int gmt_offset)
 {
-        char buf[128];
-        const char *sql_operator;
         int ret;
+        char buf[128];
+
+        if ( ! gmt_offset )
+                ret = snprintf(buf, sizeof (buf), "%s", field);
+        else
+                ret = snprintf(buf, sizeof (buf), "%s + INTERVAL '%d HOUR'", field, gmt_offset / 3600);
 
-        ret = snprintf(buf, sizeof (buf), "%s + INTERVAL '%d HOUR'", field, gmt_offset / 3600);
         if ( ret < 0 || (size_t) ret >= sizeof (buf) )
                 return preludedb_error(PRELUDEDB_ERROR_GENERIC);
 
-        sql_operator = get_operator_string(operator);
-        if ( ! sql_operator )
-                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
-
         switch ( type ) {
         case PRELUDEDB_SQL_TIME_CONSTRAINT_YEAR:
-                return prelude_string_sprintf(output, "EXTRACT(YEAR FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(YEAR FROM %s)", buf);
+
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_QUARTER:
+                return prelude_string_sprintf(output, "EXTRACT(QUARTER FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MONTH:
-                return  prelude_string_sprintf(output, "EXTRACT(MONTH FROM %s) %s %d",
-                                               buf, sql_operator, value);
+                return  prelude_string_sprintf(output, "EXTRACT(MONTH FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_YDAY:
-                return prelude_string_sprintf(output, "EXTRACT(DOY FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(DOY FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MDAY:
-                return prelude_string_sprintf(output, "EXTRACT(DAY FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(DAY FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_WDAY:
-                return prelude_string_sprintf(output, "EXTRACT(DOW FROM %s) %s %d",
-                                              buf, sql_operator, value % 7 + 1);
+                return prelude_string_sprintf(output, "(EXTRACT(ISODOW FROM %s) - 1)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_HOUR:
-                return prelude_string_sprintf(output, "EXTRACT(HOUR FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(HOUR FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_MIN:
-                return prelude_string_sprintf(output, "EXTRACT(MINUTE FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(MINUTE FROM %s)", buf);
 
         case PRELUDEDB_SQL_TIME_CONSTRAINT_SEC:
-                return prelude_string_sprintf(output, "EXTRACT(SECOND FROM %s) %s %d",
-                                              buf, sql_operator, value);
+                return prelude_string_sprintf(output, "EXTRACT(SECOND FROM %s)", buf);
+
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_MSEC:
+                return prelude_string_sprintf(output, "EXTRACT(MILLISECOND FROM %s)", buf);
+
+        case PRELUDEDB_SQL_TIME_CONSTRAINT_USEC:
+                return prelude_string_sprintf(output, "EXTRACT(MICROSECOND FROM %s)", buf);
         }
 
         /* not reached */
@@ -441,44 +544,85 @@ static int sql_build_time_constraint_string(prelude_string_t *output, const char
 
 
 
-static int sql_build_time_interval_string(preludedb_sql_time_constraint_type_t type, int value,
-                                          char *buf, size_t size)
+static int sql_build_time_timezone_string(void *session, prelude_string_t *output, const char *field, const char *tzvalue)
 {
-        char *type_str;
+        return prelude_string_sprintf(output, "timezone('%s', timezone('UTC', %s))", tzvalue, field);
+
+}
+
+
+static int sql_build_time_constraint_string(void *session, prelude_string_t *output, const char *field,
+                                            preludedb_sql_time_constraint_type_t type,
+                                            idmef_criterion_operator_t operator, int value, int gmt_offset)
+{
+        const char *sql_operator;
         int ret;
 
-        switch ( type ) {
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_YEAR:
-                type_str = "YEAR";
+        ret = sql_build_time_extract_string(session, output, field, type, gmt_offset);
+        if ( ret < 0 )
+                return ret;
+
+        sql_operator = get_operator_string(operator);
+        if ( ! sql_operator )
+                return preludedb_error(PRELUDEDB_ERROR_GENERIC);
+
+        if ( type == PRELUDEDB_SQL_TIME_CONSTRAINT_WDAY )
+                value = value % 7 + 1;
+
+        return prelude_string_sprintf(output, " %s %d", sql_operator, value);
+}
+
+
+static int sql_build_time_interval_string(void *session, prelude_string_t *output, const char *field, const char *value, preludedb_selected_object_interval_t unit)
+{
+        char *end;
+        const char *sunit;
+
+        switch ( unit ) {
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_YEAR:
+                sunit = "YEAR";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MONTH:
-                type_str = "MONTH";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_QUARTER:
+                sunit = "QUARTER";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MDAY:
-                type_str = "DAY";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_MONTH:
+                sunit = "MONTH";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_HOUR:
-                type_str = "HOUR";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_DAY:
+                sunit = "DAY";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_MIN:
-                type_str = "MINUTE";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_HOUR:
+                sunit = "HOUR";
                 break;
 
-        case PRELUDEDB_SQL_TIME_CONSTRAINT_SEC:
-                type_str = "SECOND";
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_MIN:
+                sunit = "MINUTE";
+                break;
+
+        case PRELUDEDB_SELECTED_OBJECT_INTERVAL_SEC:
+                sunit = "SECOND";
                 break;
 
         default:
                 return preludedb_error(PRELUDEDB_ERROR_GENERIC);
         }
 
-        ret = snprintf(buf, size, "INTERVAL '%d %s'", value, type_str);
+        strtol(value, &end, 10);
+        if ( end )
+                return prelude_string_sprintf(output, "(%s + (%s * INTERVAL '1 %s'))", field, value, sunit);
+        else
+                return prelude_string_sprintf(output, "(%s + INTERVAL '%s %s')", field, value, sunit);
+}
 
-        return (ret < 0 || (size_t) ret >= size) ? preludedb_error(PRELUDEDB_ERROR_GENERIC) : 0;
+
+
+static long sql_get_server_version(void *session)
+{
+        return PQserverVersion(session);
 }
 
 
@@ -501,7 +645,8 @@ int pgsql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data)
         preludedb_plugin_sql_set_escape_binary_func(plugin, sql_escape_binary);
         preludedb_plugin_sql_set_unescape_binary_func(plugin, sql_unescape_binary);
         preludedb_plugin_sql_set_query_func(plugin, sql_query);
-        preludedb_plugin_sql_set_resource_destroy_func(plugin, sql_resource_destroy);
+        preludedb_plugin_sql_set_get_server_version_func(plugin, sql_get_server_version);
+        preludedb_plugin_sql_set_table_destroy_func(plugin, sql_table_destroy);
         preludedb_plugin_sql_set_get_column_count_func(plugin, sql_get_column_count);
         preludedb_plugin_sql_set_get_row_count_func(plugin, sql_get_row_count);
         preludedb_plugin_sql_set_get_column_name_func(plugin, sql_get_column_name);
@@ -510,9 +655,12 @@ int pgsql_LTX_preludedb_plugin_init(prelude_plugin_entry_t *pe, void *data)
         preludedb_plugin_sql_set_fetch_row_func(plugin, sql_fetch_row);
         preludedb_plugin_sql_set_fetch_field_func(plugin, sql_fetch_field);
         preludedb_plugin_sql_set_build_constraint_string_func(plugin, sql_build_constraint_string);
+        preludedb_plugin_sql_set_build_time_extract_string_func(plugin, sql_build_time_extract_string);
+        preludedb_plugin_sql_set_build_time_timezone_string_func(plugin, sql_build_time_timezone_string);
         preludedb_plugin_sql_set_build_time_constraint_string_func(plugin, sql_build_time_constraint_string);
         preludedb_plugin_sql_set_build_time_interval_string_func(plugin, sql_build_time_interval_string);
         preludedb_plugin_sql_set_build_limit_offset_string_func(plugin, sql_build_limit_offset_string);
+        preludedb_plugin_sql_set_get_last_insert_ident_func(plugin, sql_get_last_insert_ident);
 
         return 0;
 }
