@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -48,6 +48,7 @@
 
 static sig_atomic_t stop_processing = 0;
 static const char *query_logging = NULL;
+static prelude_bool_t delete_run_optimize = FALSE;
 static prelude_bool_t have_query_logging = FALSE;
 
 static uint64_t cur_count = 0;
@@ -111,6 +112,14 @@ static stat_item_t *stat_item_new(const char *opname)
 
 
 
+static void stat_item_destroy(stat_item_t *st)
+{
+        prelude_list_del(&st->list);
+        free(st);
+}
+
+
+
 static void stat_dump_all(void)
 {
         stat_item_t *stat;
@@ -288,6 +297,10 @@ static int fetch_message_idents_limited(preludedb_t *db, preludedb_result_idents
         local_limit = (limit_copy < 0) ? events_per_transaction : MIN(events_per_transaction, limit_copy);
 
         count = get_message_idents(db, criteria, (int) local_limit, (int) offset + ((no_increment) ? 0 : cur_count), 0, result);
+        if ( count <= 0 )
+                return count;
+
+        count = preludedb_result_idents_get_count(*result);
         if ( count < 0 )
                 return db_error(db, count, "retrieving alert ident failed");
 
@@ -298,6 +311,14 @@ static int fetch_message_idents_limited(preludedb_t *db, preludedb_result_idents
 }
 
 
+
+static int set_delete_optimize(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
+{
+        delete_run_optimize = TRUE;
+        return 0;
+}
+
+
 static int set_criteria(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
 {
         int ret;
@@ -394,6 +415,20 @@ static void cmd_delete_help(void)
 
         fprintf(stderr, "Delete messages from <database>.\n\n");
 
+        cmd_generic_help();
+        fprintf(stderr, "  --optimize                      : Run OPTIMIZE after delete operation.\n");
+
+}
+
+
+
+static void cmd_optimize_help(void)
+{
+        fprintf(stderr, "Usage  : optimize <database>\n");
+        fprintf(stderr, "Example: preludedb-admin optimize \"type=mysql name=dbname user=prelude\"\n\n");
+
+        fprintf(stderr, "Perform optimization operation on <database>.\n\n");
+
         cmd_generic_help();
 }
 
@@ -463,15 +498,16 @@ static void cmd_count_help(void)
 
 static void print_help(char **argv)
 {
-        fprintf(stderr, "Usage: %s <count|copy|delete|load|move|print|save> <arguments>\n\n", argv[0]);
+        fprintf(stderr, "Usage: %s <count|copy|delete|load|move|print|save|optimize> <arguments>\n\n", argv[0]);
 
-        fprintf(stderr, "\tcount  - Retrieve event count from the database.\n");
-        fprintf(stderr, "\tcopy   - Make a copy of a Prelude database to another database.\n");
-        fprintf(stderr, "\tdelete - Delete content of a Prelude database.\n");
-        fprintf(stderr, "\tload   - Load a Prelude database from a file.\n");
-        fprintf(stderr, "\tmove   - Move content of a Prelude database to another database.\n");
-        fprintf(stderr, "\tprint  - Print message from a Prelude database.\n");
-        fprintf(stderr, "\tsave   - Save a Prelude database to a file.\n\n");
+        fprintf(stderr, "\tcount    - Retrieve event count from the database.\n");
+        fprintf(stderr, "\tcopy     - Make a copy of a Prelude database to another database.\n");
+        fprintf(stderr, "\tdelete   - Delete content of a Prelude database.\n");
+        fprintf(stderr, "\tload     - Load a Prelude database from a file.\n");
+        fprintf(stderr, "\tmove     - Move content of a Prelude database to another database.\n");
+        fprintf(stderr, "\tprint    - Print message from a Prelude database.\n");
+        fprintf(stderr, "\tsave     - Save a Prelude database to a file.\n");
+        fprintf(stderr, "\toptimize - Optimize a Prelude database.\n\n");
 }
 
 
@@ -598,8 +634,9 @@ static int copy_iterate(preludedb_t *src, preludedb_t *dst,
         idmef_message_t *msg;
         size_t delete_index = 0;
         uint64_t delete_tbl[1024];
+        unsigned int ident_idx = 0;
 
-        while ( (ret = preludedb_result_idents_get_next(idents, &ident)) > 0 ) {
+        while ( (ret = preludedb_result_idents_get(idents, ident_idx++, &ident)) > 0 ) {
 
                 stat_compute(stat_fetch, ret = get_message(src, ident, &msg), 1);
                 if ( ret < 0 ) {
@@ -724,6 +761,10 @@ static int cmd_delete(int argc, char **argv)
         unsigned int event_no = 0;
         preludedb_result_idents_t *idents;
         stat_item_t *stat_delete = stat_item_new("delete");
+        stat_item_t *stat_optimize = stat_item_new("optimize");
+
+        prelude_option_add(NULL, NULL, PRELUDE_OPTION_TYPE_CLI, 'o', "optimize",
+                           NULL, PRELUDE_OPTION_ARGUMENT_NONE, set_delete_optimize, NULL);
 
         idx = setup_generic_options(&argc, argv);
         if ( idx < 0 || argc != 3 ) {
@@ -742,6 +783,9 @@ static int cmd_delete(int argc, char **argv)
                 return -1;
         }
 
+        if ( ! delete_run_optimize )
+                fprintf(stderr, "WARNING: Orphan data may remain. Please run OPTIMIZE command to complete the process.\n");
+
         ret = db_new_from_string(&db, argv[idx]);
         if ( ret < 0 )
                 return ret;
@@ -761,12 +805,52 @@ static int cmd_delete(int argc, char **argv)
 
         transaction_end(db, ret, event_no);
 
+        if ( ret >= 0 && delete_run_optimize ) {
+                stat_compute(stat_optimize, ret = preludedb_optimize(db), 1);
+
+                if ( ret < 0 ) {
+                        if ( prelude_error_get_code(ret) != PRELUDE_ERROR_ENOSYS )
+                                db_error(db, ret, "error running optimize");
+                        else {
+                                ret = 0;
+                                stat_item_destroy(stat_optimize);
+                        }
+                }
+        }
+
         preludedb_destroy(db);
         return ret;
 }
 
 
 
+static int cmd_optimize(int argc, char **argv)
+{
+        int ret, idx;
+        preludedb_t *db;
+        stat_item_t *stat = stat_item_new("optimize");
+
+        idx = setup_generic_options(&argc, argv);
+        if ( idx < 0 || argc != 2 ) {
+                cmd_optimize_help();
+                exit(1);
+        }
+
+        ret = db_new_from_string(&db, argv[idx]);
+        if ( ret < 0 )
+                return ret;
+
+        stat_compute(stat, ret = preludedb_optimize(db), 1);
+        if ( ret < 0 )
+                db_error(db, ret, "error running optimize");
+
+        preludedb_destroy(db);
+        return ret;
+}
+
+
+
+
 static int save_msg(prelude_msgbuf_t *msgbuf, prelude_msg_t *msg)
 {
         int ret;
@@ -787,8 +871,9 @@ static int save_iterate_message(preludedb_t *db, preludedb_result_idents_t *iden
         int ret = 0;
         uint64_t ident;
         idmef_message_t *message;
+        unsigned int ident_idx = 0;
 
-        while ( ! stop_processing && (ret = preludedb_result_idents_get_next(idents, &ident)) > 0 ) {
+        while ( ! stop_processing && (ret = preludedb_result_idents_get(idents, ident_idx++, &ident)) > 0 ) {
 
                 stat_compute(stat_fetch, ret = get_message(db, ident, &message), 1);
                 if ( ret < 0 ) {
@@ -1032,10 +1117,10 @@ static int print_iterate_message(preludedb_t *db, preludedb_result_idents_t *ide
 {
         int ret = 0;
         uint64_t ident;
+        unsigned int ident_idx = 0;
         idmef_message_t *idmef;
 
-        while ( ! stop_processing && (ret = preludedb_result_idents_get_next(idents, &ident)) > 0 ) {
-
+        while ( ! stop_processing && (ret = preludedb_result_idents_get(idents, ident_idx++, &ident)) > 0 ) {
                 stat_compute(stat_fetch, ret = get_message(db, ident, &idmef), 1);
                 if ( ret < 0 ) {
                         db_error(db, ret, "Error retrieving message ident %" PRELUDE_PRIu64, ident);
@@ -1117,8 +1202,9 @@ static int cmd_count(int argc, char **argv)
         char buf[128];
         int ret, idx;
         preludedb_t *db;
-        idmef_value_t **values;
-        preludedb_result_values_t *results;
+        idmef_value_t *value;
+        void *row;
+        preludedb_result_values_t *results = NULL;
         preludedb_path_selection_t *ps;
         preludedb_selected_path_t *sp;
 
@@ -1140,7 +1226,7 @@ static int cmd_count(int argc, char **argv)
         if ( ret < 0 )
                 return ret;
 
-        ret = preludedb_path_selection_new(&ps);
+        ret = preludedb_path_selection_new(db, &ps);
         if ( ret < 0 ) {
                 preludedb_destroy(db);
                 return ret;
@@ -1156,21 +1242,29 @@ static int cmd_count(int argc, char **argv)
         preludedb_path_selection_add(ps, sp);
 
         ret = preludedb_get_values(db, ps, criteria, FALSE, (int) limit, (int) offset, &results);
-        if ( ret < 0 ) {
-                ret = db_error(db, ret, "error retrieving database count");
+        if ( ret <= 0 ) {
+                if ( ret == 0 )
+                        ret = db_error(db, ret, "error retrieving database count");
                 goto err;
         }
 
-        ret = preludedb_result_values_get_next(results, &values);
+        ret = preludedb_result_values_get_row(results, 0, &row);
         if ( ret < 0 )
                 goto err;
 
-        printf("COUNT: %u\n", idmef_value_get_uint32(values[0]));
-        idmef_value_destroy(values[0]);
+        ret = preludedb_result_values_get_field(results, row, sp, &value);
+        if ( ret < 0 )
+                return ret;
+
+        printf("COUNT: %u\n", idmef_value_get_uint32(value));
+        idmef_value_destroy(value);
 
 err:
         preludedb_path_selection_destroy(ps);
-        preludedb_result_values_destroy(results);
+
+        if ( results )
+                preludedb_result_values_destroy(results);
+
         preludedb_destroy(db);
 
         return ret;
@@ -1178,6 +1272,92 @@ err:
 
 
 
+static char *update_order = NULL;
+
+static int set_order(prelude_option_t *opt, const char *optarg, prelude_string_t *err, void *context)
+{
+        if ( update_order )
+                free(update_order);
+
+        update_order = strdup(optarg);
+        return 0;
+}
+
+
+
+static int cmd_update(int argc, char **argv)
+{
+        int ret, idx, argc2;
+        preludedb_t *db;
+        int i, j = 0;
+        idmef_path_t **paths;
+        idmef_value_t **values;
+        preludedb_path_selection_t *selection = NULL;
+
+        prelude_option_add(NULL, NULL, PRELUDE_OPTION_TYPE_CLI, 0, "order",
+                           NULL, PRELUDE_OPTION_ARGUMENT_REQUIRED, set_order, NULL);
+
+        argc2 = argc;
+        idx = setup_generic_options(&argc2, argv);
+        if ( idx < 0 || argc < 3 ) {
+                cmd_count_help();
+                exit(1);
+        }
+
+        ret = db_new_from_string(&db, argv[idx++]);
+        if ( ret < 0 )
+                return ret;
+
+        if ( update_order ) {
+                char *ptr;
+                preludedb_selected_path_t *selected;
+
+                preludedb_path_selection_new(db, &selection);
+
+                while ( (ptr = strsep(&update_order, ",")) ) {
+                        ret = preludedb_selected_path_new_string(&selected, ptr);
+                        if ( ret < 0 ) {
+                                fprintf(stderr, "Invalid update order path: '%s'.\n", preludedb_strerror(ret));
+                                return ret;
+                        }
+
+                        preludedb_path_selection_add(selection, selected);
+                }
+        }
+
+        paths = malloc((argc - idx) * sizeof(idmef_path_t *));
+        values = malloc((argc - idx) * sizeof(idmef_value_t *));
+
+        for ( i = 0, j = 0; (i + 1) < (argc - idx); i += 2, j++ ) {
+                ret = idmef_path_new_fast(&paths[j], argv[idx + i]);
+                if ( ret < 0 )
+                        goto err;
+
+                ret = idmef_value_new_from_path(&values[j], paths[j], argv[idx + i + 1]);
+                if ( ret < 0 )
+                        goto err;
+        }
+
+        ret = preludedb_update(db, (const idmef_path_t * const *) paths, (const idmef_value_t * const *) values,
+                               (argc - idx) / 2, criteria, selection, (int) limit, (int) offset);
+        free(paths);
+
+        for ( i = 0; i < (argc - idx) / 2; i++ )
+                free(values[i]);
+        free(values);
+
+        if ( ret < 0 ) {
+                ret = db_error(db, ret, "error performing update commande");
+                goto err;
+        }
+
+err:
+        preludedb_destroy(db);
+        return ret;
+}
+
+
+
 int main(int argc, char **argv)
 {
         int ret;
@@ -1186,13 +1366,15 @@ int main(int argc, char **argv)
                 char *name;
                 int (*run)(int argc, char **argv);
         } commands[] = {
-                { "count", cmd_count   },
-                { "copy", cmd_copy     },
-                { "delete", cmd_delete },
-                { "load", cmd_load     },
-                { "move", cmd_move     },
-                { "print", cmd_print   },
-                { "save", cmd_save     },
+                { "count", cmd_count       },
+                { "copy", cmd_copy         },
+                { "delete", cmd_delete     },
+                { "load", cmd_load         },
+                { "move", cmd_move         },
+                { "print", cmd_print       },
+                { "save", cmd_save         },
+                { "update", cmd_update     },
+                { "optimize", cmd_optimize },
         };
 
         signal(SIGINT, handle_signal);
