@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -50,7 +50,8 @@ const char *preludedb_strerror(preludedb_error_t error)
                 "Database schema version too recent",
                 "Database schema version invalid",
                 "Cannot load sql plugin",
-                "Cannot load format plugin"
+                "Cannot load format plugin",
+                "Invalid index"
         };
 
         if ( prelude_error_is_verbose(error) )
@@ -61,7 +62,7 @@ const char *preludedb_strerror(preludedb_error_t error)
 
                 code = prelude_error_get_code(error);
                 if ( code >= sizeof(error_strings) / sizeof(*error_strings) )
-                        return NULL;
+                        return "Unknown error code";
 
                 return error_strings[code];
         }
