@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2005-2017 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,12 +15,14 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
+#include "config.h"
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -29,11 +31,7 @@
 #include "preludedb-plugin-sql.h"
 
 
-#if ! defined(ENOTSUP) && defined(EOPNOTSUPP)
-# define ENOTSUP EOPNOTSUPP
-#endif
-
-#define PRELUDEDB_ENOTSUP(x) preludedb_error_verbose(prelude_error_code_from_errno(ENOTSUP), "Database backend does not support '%s' operation", x)
+#define PRELUDEDB_ENOTSUP(x) preludedb_error_verbose(prelude_error_code_from_errno(ENOSYS), "Database backend does not support '%s' operation", x)
 
 
 struct preludedb_plugin_sql {
@@ -49,16 +47,21 @@ struct preludedb_plugin_sql {
         preludedb_plugin_sql_get_row_count_func_t get_row_count;
         preludedb_plugin_sql_get_column_name_func_t get_column_name;
         preludedb_plugin_sql_get_column_num_func_t get_column_num;
-        preludedb_plugin_sql_resource_destroy_func_t resource_destroy;
+        preludedb_plugin_sql_field_destroy_func_t field_destroy;
+        preludedb_plugin_sql_row_destroy_func_t row_destroy;
+        preludedb_plugin_sql_table_destroy_func_t table_destroy;
         preludedb_plugin_sql_fetch_row_func_t fetch_row;
         preludedb_plugin_sql_fetch_field_func_t fetch_field;
+        preludedb_plugin_sql_build_time_extract_string_func_t build_time_extract_string;
         preludedb_plugin_sql_build_time_constraint_string_func_t build_time_constraint_string;
         preludedb_plugin_sql_build_time_interval_string_func_t build_time_interval_string;
         preludedb_plugin_sql_build_limit_offset_string_func_t build_limit_offset_string;
         preludedb_plugin_sql_build_constraint_string_func_t build_constraint_string;
         preludedb_plugin_sql_get_operator_string_func_t get_operator_string;
         preludedb_plugin_sql_build_timestamp_string_func_t build_timestamp_string;
-
+        preludedb_plugin_sql_get_server_version_func_t get_server_version;
+        preludedb_plugin_sql_get_last_insert_ident_func_t get_last_insert_ident;
+        preludedb_plugin_sql_build_time_timezone_string_func_t build_time_timezone_string;
 };
 
 
@@ -173,13 +176,31 @@ int _preludedb_plugin_sql_unescape_binary(preludedb_plugin_sql_t *plugin, void *
 }
 
 
+void preludedb_plugin_sql_set_get_last_insert_ident_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_get_last_insert_ident_func_t func)
+{
+        plugin->get_last_insert_ident = func;
+}
+
+
+int _preludedb_plugin_sql_get_last_insert_ident(preludedb_plugin_sql_t *plugin, void *session, uint64_t *ident)
+{
+        if ( ! plugin->get_last_insert_ident )
+                return PRELUDEDB_ENOTSUP("get_last_insert_ident");
+
+        if ( ! session )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "could not retrieve last insert ID: session not initialized");
+
+        return plugin->get_last_insert_ident(session, ident);
+}
+
+
 void preludedb_plugin_sql_set_query_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_query_func_t func)
 {
         plugin->query = func;
 }
 
 
-int _preludedb_plugin_sql_query(preludedb_plugin_sql_t *plugin, void *session, const char *query, void **res)
+int _preludedb_plugin_sql_query(preludedb_plugin_sql_t *plugin, void *session, const char *query, preludedb_sql_table_t **res)
 {
         return plugin->query(session, query, res);
 }
@@ -191,9 +212,9 @@ void preludedb_plugin_sql_set_get_column_count_func(preludedb_plugin_sql_t *plug
 }
 
 
-unsigned int _preludedb_plugin_sql_get_column_count(preludedb_plugin_sql_t *plugin, void *session, void *resource)
+unsigned int _preludedb_plugin_sql_get_column_count(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table)
 {
-        return plugin->get_column_count(session, resource);
+        return plugin->get_column_count(session, table);
 }
 
 
@@ -203,9 +224,12 @@ void preludedb_plugin_sql_set_get_row_count_func(preludedb_plugin_sql_t *plugin,
 }
 
 
-unsigned int _preludedb_plugin_sql_get_row_count(preludedb_plugin_sql_t *plugin, void *session, void *resource)
+unsigned int _preludedb_plugin_sql_get_row_count(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table)
 {
-        return plugin->get_row_count(session, resource);
+        if ( ! plugin->get_row_count )
+                return PRELUDEDB_ENOTSUP("get_row_count");
+
+        return plugin->get_row_count(session, table);
 }
 
 
@@ -215,9 +239,9 @@ void preludedb_plugin_sql_set_get_column_name_func(preludedb_plugin_sql_t *plugi
 }
 
 
-const char *_preludedb_plugin_sql_get_column_name(preludedb_plugin_sql_t *plugin, void *session, void *resource, unsigned int column_num)
+const char *_preludedb_plugin_sql_get_column_name(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table, unsigned int column_num)
 {
-        return plugin->get_column_name(session, resource, column_num);
+        return plugin->get_column_name(session, table, column_num);
 }
 
 
@@ -227,21 +251,47 @@ void preludedb_plugin_sql_set_get_column_num_func(preludedb_plugin_sql_t *plugin
 }
 
 
-int _preludedb_plugin_sql_get_column_num(preludedb_plugin_sql_t *plugin, void *session, void *resource, const char *column_name)
+int _preludedb_plugin_sql_get_column_num(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table, const char *column_name)
 {
-        return plugin->get_column_num(session, resource, column_name);
+        return plugin->get_column_num(session, table, column_name);
 }
 
 
-void preludedb_plugin_sql_set_resource_destroy_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_resource_destroy_func_t func)
+void preludedb_plugin_sql_set_field_destroy_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_field_destroy_func_t func)
 {
-        plugin->resource_destroy = func;
+        plugin->field_destroy = func;
 }
 
 
-void _preludedb_plugin_sql_resource_destroy(preludedb_plugin_sql_t *plugin, void *session, void *resource)
+void _preludedb_plugin_sql_field_destroy(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table, preludedb_sql_row_t *row, preludedb_sql_field_t *field)
 {
-        plugin->resource_destroy(session, resource);
+        if ( plugin->field_destroy )
+                plugin->field_destroy(session, table, row, field);
+}
+
+
+void preludedb_plugin_sql_set_row_destroy_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_row_destroy_func_t func)
+{
+        plugin->row_destroy = func;
+}
+
+
+void _preludedb_plugin_sql_row_destroy(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table, preludedb_sql_row_t *row)
+{
+        if ( plugin->row_destroy )
+                plugin->row_destroy(session, table, row);
+}
+
+
+void preludedb_plugin_sql_set_table_destroy_func(preludedb_plugin_sql_t *plugin, preludedb_plugin_sql_table_destroy_func_t func)
+{
+        plugin->table_destroy = func;
+}
+
+
+void _preludedb_plugin_sql_table_destroy(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table)
+{
+        plugin->table_destroy(session, table);
 }
 
 
@@ -251,12 +301,12 @@ void preludedb_plugin_sql_set_fetch_row_func(preludedb_plugin_sql_t *plugin, pre
 }
 
 
-int _preludedb_plugin_sql_fetch_row(preludedb_plugin_sql_t *plugin, void *session, void *resource, void **row)
+int _preludedb_plugin_sql_fetch_row(preludedb_plugin_sql_t *plugin, void *session, preludedb_sql_table_t *table, unsigned int row_index, preludedb_sql_row_t **row)
 {
         if ( ! plugin->fetch_row )
-                PRELUDEDB_ENOTSUP("fetch_row");
+                return PRELUDEDB_ENOTSUP("fetch_row");
 
-        return plugin->fetch_row(session, resource, row);
+        return plugin->fetch_row(session, table, row_index, row);
 }
 
 
@@ -267,13 +317,52 @@ void preludedb_plugin_sql_set_fetch_field_func(preludedb_plugin_sql_t *plugin, p
 
 
 int _preludedb_plugin_sql_fetch_field(preludedb_plugin_sql_t *plugin,
-                                      void *session, void *resource, void *row,
-                                      unsigned int column_num, const char **value, size_t *len)
+                                      void *session, preludedb_sql_table_t *table, void *row,
+                                      unsigned int column_num, preludedb_sql_field_t **field)
 {
         if ( ! plugin->fetch_field )
                 return PRELUDEDB_ENOTSUP("fetch_field");
 
-        return plugin->fetch_field(session, resource, row, column_num, value, len);
+        return plugin->fetch_field(session, table, row, column_num, field);
+}
+
+
+void preludedb_plugin_sql_set_build_time_extract_string_func(preludedb_plugin_sql_t *plugin,
+                                                             preludedb_plugin_sql_build_time_extract_string_func_t func)
+{
+        plugin->build_time_extract_string = func;
+}
+
+
+
+int _preludedb_plugin_sql_build_time_extract_string(preludedb_plugin_sql_t *plugin, void *session,
+                                                    prelude_string_t *output, const char *field,
+                                                    preludedb_sql_time_constraint_type_t type,
+                                                   int gmt_offset)
+{
+        if ( ! plugin->build_time_extract_string )
+                return PRELUDEDB_ENOTSUP("build_time_extract_string");
+
+        return plugin->build_time_extract_string(session, output, field, type, gmt_offset);
+}
+
+
+
+void preludedb_plugin_sql_set_build_time_timezone_string_func(preludedb_plugin_sql_t *plugin,
+                                                              preludedb_plugin_sql_build_time_timezone_string_func_t func)
+{
+        plugin->build_time_timezone_string = func;
+}
+
+
+
+int _preludedb_plugin_sql_build_time_timezone_string(preludedb_plugin_sql_t *plugin, void *session,
+                                                     prelude_string_t *output, const char *field, const char *tzvalue)
+{
+        if ( ! plugin->build_time_timezone_string )
+                return PRELUDEDB_ENOTSUP("build_time_timezone_string");
+
+        return plugin->build_time_timezone_string(session, output, field, tzvalue);
 }
 
 
@@ -284,7 +373,7 @@ void preludedb_plugin_sql_set_build_time_constraint_string_func(preludedb_plugin
 }
 
 
-int _preludedb_plugin_sql_build_time_constraint_string(preludedb_plugin_sql_t *plugin,
+int _preludedb_plugin_sql_build_time_constraint_string(preludedb_plugin_sql_t *plugin, void *session,
                                                        prelude_string_t *output, const char *field,
                                                        preludedb_sql_time_constraint_type_t type,
                                                        idmef_criterion_operator_t operator, int value, int gmt_offset)
@@ -292,7 +381,7 @@ int _preludedb_plugin_sql_build_time_constraint_string(preludedb_plugin_sql_t *p
         if ( ! plugin->build_time_constraint_string )
                 return PRELUDEDB_ENOTSUP("build_time_constraint_string");
 
-        return plugin->build_time_constraint_string(output, field, type, operator, value, gmt_offset);
+        return plugin->build_time_constraint_string(session, output, field, type, operator, value, gmt_offset);
 }
 
 
@@ -304,13 +393,12 @@ void preludedb_plugin_sql_set_build_time_interval_string_func(preludedb_plugin_s
 }
 
 
-int _preludedb_plugin_sql_build_time_interval_string(preludedb_plugin_sql_t *plugin,
-                                                     preludedb_sql_time_constraint_type_t type, int value, char *buf, size_t size)
+int _preludedb_plugin_sql_build_time_interval_string(preludedb_plugin_sql_t *plugin,  void *session, prelude_string_t *output, const char *field, const char *value, preludedb_selected_object_interval_t unit)
 {
         if ( ! plugin->build_time_interval_string )
                 return PRELUDEDB_ENOTSUP("build_time_interval_string");
 
-        return plugin->build_time_interval_string(type, value, buf, size);
+        return plugin->build_time_interval_string(session, output, field, value, unit);
 }
 
 
@@ -338,14 +426,14 @@ void preludedb_plugin_sql_set_build_constraint_string_func(preludedb_plugin_sql_
 }
 
 
-int _preludedb_plugin_sql_build_constraint_string(preludedb_plugin_sql_t *plugin,
+int _preludedb_plugin_sql_build_constraint_string(preludedb_plugin_sql_t *plugin, void *session,
                                                   prelude_string_t *out, const char *field,
                                                   idmef_criterion_operator_t operator, const char *value)
 {
         if ( ! plugin->build_constraint_string )
                 return PRELUDEDB_ENOTSUP("build_constraint_string");
 
-        return plugin->build_constraint_string(out, field, operator, value);
+        return plugin->build_constraint_string(session, out, field, operator, value);
 }
 
 
@@ -368,12 +456,12 @@ void preludedb_plugin_sql_set_build_timestamp_string_func(preludedb_plugin_sql_t
 }
 
 
-int _preludedb_plugin_sql_build_timestamp_string(preludedb_plugin_sql_t *plugin, const struct tm *lt, char *out, size_t size)
+int _preludedb_plugin_sql_build_timestamp_string(preludedb_plugin_sql_t *plugin, void *session, const struct tm *lt, char *out, size_t size)
 {
         int ret;
 
         if ( plugin->build_timestamp_string )
-                return plugin->build_timestamp_string(lt, out, size);
+                return plugin->build_timestamp_string(session, lt, out, size);
 
         ret = snprintf(out, size, "'%d-%.2d-%.2d %.2d:%.2d:%.2d'",
                        lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
@@ -383,6 +471,21 @@ int _preludedb_plugin_sql_build_timestamp_string(preludedb_plugin_sql_t *plugin,
 }
 
 
+void preludedb_plugin_sql_set_get_server_version_func(preludedb_plugin_sql_t *plugin,
+                                                      preludedb_plugin_sql_get_server_version_func_t func)
+{
+        plugin->get_server_version = func;
+}
+
+
+long _preludedb_plugin_sql_get_server_version(preludedb_plugin_sql_t *plugin, void *session)
+{
+        if ( ! plugin->get_server_version )
+                return PRELUDEDB_ENOTSUP("get_server_version");
+
+        return plugin->get_server_version(session);
+}
+
 
 int preludedb_plugin_sql_new(preludedb_plugin_sql_t **plugin)
 {
