@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -62,6 +62,13 @@
 #include "preludedb.h"
 
 
+#ifndef MAX
+# define MAX(x, y) (((x) > (y)) ? (x) : (y))
+#endif
+
+#define SQL_NULL_FIELD (void *) 0xdeadbeef
+
+
 typedef enum {
         PRELUDEDB_SQL_STATUS_CONNECTED    = 0x01,
         PRELUDEDB_SQL_STATUS_TRANSACTION  = 0x02
@@ -89,32 +96,41 @@ struct preludedb_sql {
         void *session;
         FILE *logfile;
         prelude_bool_t internal_transaction_disabled;
-
         gl_recursive_lock_t mutex;
+        int refcount;
 };
 
 struct preludedb_sql_table {
         preludedb_sql_t *sql;
-        void *res;
-        prelude_list_t row_list;
+        void *data;
+
+        preludedb_sql_row_t **rows;
+        unsigned int nrow;
+        unsigned int row_count;
+        unsigned int column_count;
+
+        uint16_t refcount;
+        uint8_t done;
 };
 
-struct preludedb_sql_row {
-        prelude_list_t list;
-        preludedb_sql_table_t *table;
-        void *res;
-        prelude_list_t field_list;
-};
 
 struct preludedb_sql_field {
-        prelude_list_t list;
-        preludedb_sql_row_t *row;
-        int num;
-        const char *value;
-        size_t len;
+        char *value;
+        uint32_t len;
+        uint32_t index;
 };
 
 
+struct preludedb_sql_row {
+        preludedb_sql_table_t *table;
+        void *data;
+        uint32_t index;
+        uint32_t refcount;
+        preludedb_sql_field_t fields[1];
+};
+
+
+
 int _preludedb_sql_transaction_start(preludedb_sql_t *sql);
 int _preludedb_sql_transaction_end(preludedb_sql_t *sql);
 int _preludedb_sql_transaction_abort(preludedb_sql_t *sql);
@@ -125,6 +141,10 @@ void _preludedb_sql_disable_internal_transaction(preludedb_sql_t *sql);
 extern prelude_list_t _sql_plugin_list;
 
 
+static inline preludedb_sql_row_t *field2row(preludedb_sql_field_t *field)
+{
+        return (preludedb_sql_row_t *) ((unsigned char *) field - (sizeof(*field) * field->index + offsetof(preludedb_sql_row_t, fields)));
+}
 
 
 static inline void update_sql_from_errno(preludedb_sql_t *sql, preludedb_error_t error)
@@ -154,12 +174,15 @@ int preludedb_sql_new(preludedb_sql_t **new, const char *type, preludedb_sql_set
         if ( ! *new )
                 return preludedb_error_from_errno(errno);
 
+        (*new)->refcount = 1;
         gl_recursive_lock_init(((*new)->mutex));
 
         if ( ! type ) {
                 type = preludedb_sql_settings_get_type(settings);
-                if ( ! type )
+                if ( ! type ) {
+                        free(*new);
                         return preludedb_error_verbose(PRELUDEDB_ERROR_INVALID_SETTINGS_STRING, "No database type specified");
+                }
         }
 
         (*new)->type = strdup(type);
@@ -185,6 +208,15 @@ int preludedb_sql_new(preludedb_sql_t **new, const char *type, preludedb_sql_set
 
 
 
+preludedb_sql_t *preludedb_sql_ref(preludedb_sql_t *sql)
+{
+        sql->refcount++;
+        return sql;
+}
+
+
+
+
 /**
  * preludedb_sql_destroy:
  * @sql: Pointer to a sql object.
@@ -193,6 +225,9 @@ int preludedb_sql_new(preludedb_sql_t **new, const char *type, preludedb_sql_set
  */
 void preludedb_sql_destroy(preludedb_sql_t *sql)
 {
+        if ( --sql->refcount > 0 )
+                return;
+
         if ( sql->status & PRELUDEDB_SQL_STATUS_CONNECTED )
                 _preludedb_plugin_sql_close(sql->plugin, sql->session);
 
@@ -275,21 +310,32 @@ static int preludedb_sql_connect(preludedb_sql_t *sql)
 
 
 
-static int preludedb_sql_table_new(preludedb_sql_table_t **new, preludedb_sql_t *sql, void *res)
+int preludedb_sql_table_new(preludedb_sql_table_t **new, void *data)
 {
         *new = malloc(sizeof(**new));
         if ( ! *new )
                 return preludedb_error_from_errno(errno);
 
-        (*new)->sql = sql;
-        (*new)->res = res;
-        prelude_list_init(&(*new)->row_list);
+        (*new)->rows = NULL;
+        (*new)->nrow = 0;
+        (*new)->row_count = 0;
+        (*new)->column_count = 0;
+        (*new)->done = FALSE;
+        (*new)->refcount = 1;
+        (*new)->data = data;
 
         return 0;
 }
 
 
 
+void *preludedb_sql_table_get_data(preludedb_sql_table_t *table)
+{
+        return table->data;
+}
+
+
+
 /**
  * preludedb_sql_query:
  * @sql: Pointer to a sql object.
@@ -299,12 +345,11 @@ static int preludedb_sql_table_new(preludedb_sql_table_t **new, preludedb_sql_t
  *
  * Execute a SQL query.
  *
- * Returns: the number of result or a negative value if an error occured.
+ * Returns: 1 if result are available, 0 for no result, -1 if an error occured.
  */
 int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_table_t **table)
 {
         int ret;
-        void *res;
         struct timeval start, end;
 
         gl_recursive_lock_lock(sql->mutex);
@@ -312,11 +357,19 @@ int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_t
 
         gettimeofday(&start, NULL);
 
-        ret = _preludedb_plugin_sql_query(sql->plugin, sql->session, query, &res);
-
-        if ( ret < 0 )
+        ret = _preludedb_plugin_sql_query(sql->plugin, sql->session, query, table);
+        if ( ret < 0 ) {
                 update_sql_from_errno(sql, ret);
 
+                if ( ! (sql->status & PRELUDEDB_SQL_STATUS_CONNECTED) ) {
+                        assert_connected(sql);
+
+                        ret = _preludedb_plugin_sql_query(sql->plugin, sql->session, query, table);
+                        if ( ret < 0 )
+                                update_sql_from_errno(sql, ret);
+                }
+        }
+
         gettimeofday(&end, NULL);
         gl_recursive_lock_unlock(sql->mutex);
 
@@ -331,13 +384,8 @@ int preludedb_sql_query(preludedb_sql_t *sql, const char *query, preludedb_sql_t
         if ( ret <= 0 )
                 return ret;
 
-        ret = preludedb_sql_table_new(table, sql, res);
-        if ( ret < 0 ) {
-                _preludedb_plugin_sql_resource_destroy(sql->plugin, sql->session, res);
-                return ret;
-        }
-
-        return preludedb_sql_table_get_row_count(*table);
+        (*table)->sql = preludedb_sql_ref(sql);
+        return 1;
 }
 
 
@@ -429,6 +477,26 @@ int preludedb_sql_insert(preludedb_sql_t *sql, const char *table, const char *fi
 
 
 
+
+/**
+ * preludedb_sql_get_last_insert_ident:
+ * @sql: Pointer to a sql object.
+ * @ident: Where the retrieved ident is stored
+ *
+ * Return the last insert ident allocated by the database server when
+ * writing to a table with an auto increment field.
+ *
+ * Returns: 0 on success or a negative value if an error occurs.
+ */
+int preludedb_sql_get_last_insert_ident(preludedb_sql_t *sql, uint64_t *ident)
+{
+        return _preludedb_plugin_sql_get_last_insert_ident(sql->plugin, sql->session, ident);
+}
+
+
+
+
+
 /**
  * preludedb_sql_build_limit_offset_string:
  * @sql: Pointer to a sql object.
@@ -595,6 +663,11 @@ int preludedb_sql_escape_fast(preludedb_sql_t *sql, const char *input, size_t in
 {
         int ret;
 
+        if ( ! input ) {
+                *output = (char *) strdup("NULL");
+                return *output ? 0 : preludedb_error_from_errno(errno);
+        }
+
         gl_recursive_lock_lock(sql->mutex);
 
         assert_connected(sql);
@@ -619,12 +692,7 @@ int preludedb_sql_escape_fast(preludedb_sql_t *sql, const char *input, size_t in
  */
 int preludedb_sql_escape(preludedb_sql_t *sql, const char *input, char **output)
 {
-        if ( ! input ) {
-                *output = strdup("NULL");
-                return *output ? 0 : preludedb_error_from_errno(errno);
-        }
-
-        return preludedb_sql_escape_fast(sql, input, strlen(input), output);
+        return preludedb_sql_escape_fast(sql, input, (input) ? strlen(input) : 0, output);
 }
 
 
@@ -645,6 +713,11 @@ int preludedb_sql_escape_binary(preludedb_sql_t *sql, const unsigned char *input
 {
         int ret;
 
+        if ( ! input ) {
+                *output = (char *) strdup("NULL");
+                return *output ? 0 : preludedb_error_from_errno(errno);
+        }
+
         gl_recursive_lock_lock(sql->mutex);
 
         assert_connected(sql);
@@ -686,6 +759,16 @@ int preludedb_sql_unescape_binary(preludedb_sql_t *sql, const char *input, size_
 
 
 
+preludedb_sql_table_t *preludedb_sql_table_ref(preludedb_sql_table_t *table)
+{
+        prelude_return_val_if_fail(table, NULL);
+
+        table->refcount++;
+        return table;
+}
+
+
+
 /**
  * preludedb_sql_table_destroy:
  * @table: Pointer to a table object.
@@ -694,64 +777,146 @@ int preludedb_sql_unescape_binary(preludedb_sql_t *sql, const char *input, size_
  */
 void preludedb_sql_table_destroy(preludedb_sql_table_t *table)
 {
-        preludedb_sql_t *sql = table->sql;
-        preludedb_sql_row_t *row;
-        preludedb_sql_field_t *field;
-        prelude_list_t *tmp, *next_row, *next_field;
+        unsigned int i;
 
-        prelude_list_for_each_safe(&table->row_list, tmp, next_row) {
-                row = prelude_list_entry(tmp, preludedb_sql_row_t, list);
+        if ( --table->refcount > 0 )
+                return;
 
-                prelude_list_for_each_safe(&row->field_list, tmp, next_field) {
-                        field = prelude_list_entry(tmp, preludedb_sql_field_t, list);
-                        free(field);
-                }
+        for ( i = 0; i < table->nrow; i++ )
+                if ( table->rows[i] )
+                        preludedb_sql_row_destroy(table->rows[i]);
 
-                free(row);
-        }
+        free(table->rows);
 
-        _preludedb_plugin_sql_resource_destroy(sql->plugin, sql->session, table->res);
+        _preludedb_plugin_sql_table_destroy(table->sql->plugin, table->sql->session, table);
+        preludedb_sql_destroy(table->sql);
         free(table);
 }
 
 
 
-static int preludedb_sql_row_new(preludedb_sql_row_t **row, preludedb_sql_table_t *table, void *res)
+int preludedb_sql_table_new_row(preludedb_sql_table_t *table, preludedb_sql_row_t **row, unsigned int row_index)
 {
-        *row = malloc(sizeof(**row));
+        unsigned int i;
+        unsigned int nindex = MAX(row_index, table->nrow) + 1;
+        size_t fieldsize = preludedb_sql_table_get_column_count(table) * sizeof(preludedb_sql_field_t);
+
+        if ( row_index >= table->nrow ) {
+
+                table->rows = realloc(table->rows, sizeof(*table->rows) * nindex);
+                if ( ! table->rows )
+                        return preludedb_error_from_errno(errno);
+
+                for ( i = table->nrow; i < nindex; i++ )
+                        table->rows[i] = NULL;
+
+                table->nrow = nindex;
+        }
+
+        *row = table->rows[row_index] = calloc(1, offsetof(preludedb_sql_row_t, fields) + fieldsize);
         if ( ! *row )
                 return preludedb_error_from_errno(errno);
 
+        (*row)->refcount = 1;
         (*row)->table = table;
-        (*row)->res = res;
-
-        prelude_list_init(&(*row)->list);
-        prelude_list_init(&(*row)->field_list);
-
-        prelude_list_add_tail(&table->row_list, &(*row)->list);
+        (*row)->index = row_index;
 
         return 0;
 }
 
 
-
-static int preludedb_sql_field_new(preludedb_sql_field_t **field,
-                                   preludedb_sql_row_t *row, int num, const char *value, size_t len)
+void preludedb_sql_row_set_data(preludedb_sql_row_t *row, void *data)
 {
-        *field = malloc(sizeof(**field));
-        if ( ! field )
-                return preludedb_error_from_errno(errno);
+        row->data = data;
+}
 
-        (*field)->row = row;
-        (*field)->num = num;
-        (*field)->value = value;
-        (*field)->len = len;
 
-        prelude_list_init(&(*field)->list);
+void *preludedb_sql_row_get_data(preludedb_sql_row_t *row)
+{
+        return row->data;
+}
 
-        prelude_list_add_tail(&row->field_list, &(*field)->list);
 
-        return 0;
+unsigned int preludedb_sql_row_get_field_count(preludedb_sql_row_t *row)
+{
+        return preludedb_sql_table_get_column_count(row->table);
+}
+
+
+preludedb_sql_row_t *preludedb_sql_row_ref(preludedb_sql_row_t *row)
+{
+        if ( row->refcount == 1 )
+                preludedb_sql_table_ref(row->table);
+
+        row->refcount++;
+        return row;
+}
+
+
+
+void preludedb_sql_row_destroy(preludedb_sql_row_t *row)
+{
+        unsigned int i;
+
+        if ( --row->refcount > 0 ) {
+                if ( row->refcount == 1 )
+                        preludedb_sql_table_destroy(row->table);
+                return;
+        }
+
+        _preludedb_plugin_sql_row_destroy(row->table->sql->plugin, row->table->sql->session, row->table, row);
+
+        for ( i = 0; i < preludedb_sql_table_get_column_count(row->table); i++ ) {
+                if ( row->fields[i].value )
+                        preludedb_sql_field_destroy(&(row->fields[i]));
+        }
+
+        row->table->rows[row->index] = NULL;
+        free(row);
+}
+
+
+int preludedb_sql_row_new_field(preludedb_sql_row_t *row, preludedb_sql_field_t **field,
+                                int num, char *value, size_t len)
+{
+        preludedb_sql_field_t *ftbl = row->fields;
+
+        if ( ! value ) {
+                *field = NULL;
+                ftbl[num].value = SQL_NULL_FIELD;
+                return 0;
+        }
+
+        ftbl[num].index = num;
+        ftbl[num].value = value;
+        ftbl[num].len = len;
+        *field = &ftbl[num];
+
+        return 1;
+}
+
+
+
+preludedb_sql_field_t *preludedb_sql_field_ref(preludedb_sql_field_t *field)
+{
+        preludedb_sql_row_ref(field2row(field));
+        return field;
+}
+
+
+void preludedb_sql_field_destroy(preludedb_sql_field_t *field)
+{
+        preludedb_sql_row_t *row;
+
+        if ( field->value == SQL_NULL_FIELD )
+                return;
+
+        row = field2row(field);
+
+        if ( row->refcount == 0 )
+                _preludedb_plugin_sql_field_destroy(row->table->sql->plugin, row->table->sql->session, row->table, row, field);
+        else
+                preludedb_sql_row_destroy(row);
 }
 
 
@@ -767,7 +932,7 @@ static int preludedb_sql_field_new(preludedb_sql_field_t **field,
  */
 const char *preludedb_sql_table_get_column_name(preludedb_sql_table_t *table, unsigned int column_num)
 {
-        return _preludedb_plugin_sql_get_column_name(table->sql->plugin, table->sql->session, table->res, column_num);
+        return _preludedb_plugin_sql_get_column_name(table->sql->plugin, table->sql->session, table, column_num);
 }
 
 
@@ -783,7 +948,7 @@ const char *preludedb_sql_table_get_column_name(preludedb_sql_table_t *table, un
  */
 int preludedb_sql_table_get_column_num(preludedb_sql_table_t *table, const char *column_name)
 {
-        return _preludedb_plugin_sql_get_column_num(table->sql->plugin, table->sql->session, table->res, column_name);
+        return _preludedb_plugin_sql_get_column_num(table->sql->plugin, table->sql->session, table, column_name);
 }
 
 
@@ -798,22 +963,109 @@ int preludedb_sql_table_get_column_num(preludedb_sql_table_t *table, const char
  */
 unsigned int preludedb_sql_table_get_column_count(preludedb_sql_table_t *table)
 {
-        return _preludedb_plugin_sql_get_column_count(table->sql->plugin, table->sql->session, table->res);
+        if ( ! table->column_count )
+                table->column_count = _preludedb_plugin_sql_get_column_count(table->sql->plugin, table->sql->session, table);
+
+        return table->column_count;
 }
 
 
 
 /**
- * preludedb_sql_table_get_column_count:
+ * preludedb_sql_table_get_row_count:
  * @table: Pointer to a table object.
  *
- * Get the the number of columns.
+ * Get the the number of row in the table.
+ * Depending on the database backend, this might require retrieving all rows.
  *
  * Returns: the number of columns.
  */
 unsigned int preludedb_sql_table_get_row_count(preludedb_sql_table_t *table)
 {
-        return _preludedb_plugin_sql_get_row_count(table->sql->plugin, table->sql->session, table->res);
+        int ret;
+        preludedb_sql_row_t *row;
+
+        if ( table->row_count )
+                return table->row_count;
+
+        ret = _preludedb_plugin_sql_get_row_count(table->sql->plugin, table->sql->session, table);
+        if ( ret >= 0 ) {
+                table->row_count = ret;
+                return ret;
+        }
+
+        else if ( ret < 0 && prelude_error_get_code(ret) != PRELUDE_ERROR_ENOSYS )
+                return ret;
+
+        prelude_log(PRELUDE_LOG_WARN, "SQL plugin '%s' emulate row-count before fetch: this is a slow operation.\n", preludedb_sql_get_type(table->sql));
+
+        do {
+                ret = preludedb_sql_table_fetch_row(table, &row);
+        } while ( ret > 0 );
+
+        table->row_count = table->nrow;
+
+        return table->row_count;
+}
+
+
+
+/**
+ * preludedb_sql_table_get_fetched_row_count:
+ * @table: Pointer to a table object.
+ *
+ * Get the the number of row already retrieved in the table.
+ *
+ * Returns: the number of columns.
+ */
+ unsigned int preludedb_sql_table_get_fetched_row_count(preludedb_sql_table_t *table)
+{
+        return table->nrow;
+}
+
+
+
+/**
+ * preludedb_sql_table_get_row:
+ * @table: Pointer to a table object.
+ * @row: Pointer to the row object where the result will be stored.
+ *
+ * Fetch the next table's row.
+ *
+ * Returns: 1 if the table returns a new row, 0 if there is no more rows to fetch or
+ * a negative value if an error occur.
+ */
+int preludedb_sql_table_get_row(preludedb_sql_table_t *table, unsigned int row_index, preludedb_sql_row_t **row)
+{
+        int ret;
+
+        if ( row_index == (unsigned int) -1 )
+                row_index = table->nrow;
+
+        if ( row_index < table->nrow && table->rows[row_index] ) {
+                *row = table->rows[row_index];
+                return 1;
+        }
+
+        if ( table->done ) {
+                if ( row_index == table->nrow )
+                        return 0;
+
+                return preludedb_error_verbose(PRELUDEDB_ERROR_INDEX, "Invalid row '%u'", row_index);
+        }
+
+        ret = _preludedb_plugin_sql_fetch_row(table->sql->plugin, table->sql->session, table, row_index, row);
+        if ( ret < 0 ) {
+                update_sql_from_errno(table->sql, ret);
+                return ret;
+        }
+
+        if ( ret == 0 ) {
+                table->done = TRUE;
+                return 0;
+        }
+
+        return 1;
 }
 
 
@@ -830,23 +1082,53 @@ unsigned int preludedb_sql_table_get_row_count(preludedb_sql_table_t *table)
  */
 int preludedb_sql_table_fetch_row(preludedb_sql_table_t *table, preludedb_sql_row_t **row)
 {
-        int ret;
-        void *res;
+        return preludedb_sql_table_get_row(table, -1, row);
+}
 
-        ret = _preludedb_plugin_sql_fetch_row(table->sql->plugin, table->sql->session, table->res, &res);
+
+
+/**
+ * preludedb_sql_row_get_field:
+ * @row: Pointer to a row object.
+ * @column_num: The column number of the field to be fetched.
+ * @field: Pointer to the field object where the result will be stored.
+ *
+ * Fetch the field of column @column_num
+ *
+ * Returns: 1 if the row returns a non-empty field, 0 if it returns an empty field, or
+ * a negative value if an error occur.
+ */
+int preludedb_sql_row_get_field(preludedb_sql_row_t *row, int column_num, preludedb_sql_field_t **field)
+{
+        int ret;
+        unsigned int ccount;
+
+        ccount = preludedb_sql_table_get_column_count(row->table);
+        if ( column_num < 0 )
+                column_num = ccount - (-column_num);
+
+        if ( column_num >= ccount )
+                return prelude_error_verbose(PRELUDEDB_ERROR_INDEX, "Attempt to access invalid column `%d` (max is `%d`)", column_num, ccount);
+
+        if ( row->fields[column_num].value ) {
+                if ( row->fields[column_num].value == SQL_NULL_FIELD ) {
+                        *field = NULL;
+                        return 0;
+                }
+
+                *field = &(row->fields[column_num]);
+                return 1;
+        }
+
+
+        ret = _preludedb_plugin_sql_fetch_field(row->table->sql->plugin,
+                                                row->table->sql->session, row->table, row, column_num, field);
         if ( ret < 0 ) {
-                update_sql_from_errno(table->sql, ret);
+                update_sql_from_errno(row->table->sql, ret);
                 return ret;
         }
 
-        if ( ret == 0 )
-                return 0;
-
-        ret = preludedb_sql_row_new(row, table, res);
-        if ( ret < 0 )
-                return ret;
-
-        return 1;
+        return ret;
 }
 
 
@@ -857,33 +1139,39 @@ int preludedb_sql_table_fetch_row(preludedb_sql_table_t *table, preludedb_sql_ro
  * @column_num: The column number of the field to be fetched.
  * @field: Pointer to the field object where the result will be stored.
  *
+ * DEPRECATED: use preludedb_sql_row_get_field() instead.
  * Fetch the field of column @column_num
  *
  * Returns: 1 if the row returns a non-empty field, 0 if it returns an empty field, or
  * a negative value if an error occur.
  */
-int preludedb_sql_row_fetch_field(preludedb_sql_row_t *row, unsigned int column_num,
-                                  preludedb_sql_field_t **field)
+int preludedb_sql_row_fetch_field(preludedb_sql_row_t *row, int column_num, preludedb_sql_field_t **field)
 {
-        const char *value;
-        size_t len;
-        int ret;
+        return preludedb_sql_row_get_field(row, column_num, field);
+}
 
-        ret = _preludedb_plugin_sql_fetch_field(row->table->sql->plugin,
-                                                row->table->sql->session, row->table->res, row->res, column_num, &value, &len);
-        if ( ret < 0 ) {
-                update_sql_from_errno(row->table->sql, ret);
-                return ret;
-        }
 
-        if ( ret == 0 )
-                return 0;
+/**
+ * preludedb_sql_row_get_field_by_name:
+ * @row: Pointer to a row object.
+ * @column_name: The column name of the field to be fetched.
+ * @field: Pointer to the field object where the result will be stored.
+ *
+ * Fetch the field of column @column_name
+ *
+ * Returns: 1 if the row returns a non-empty field, 0 if it returns an empty field, or
+ * a negative value if an error occur.
+ */
+int preludedb_sql_row_get_field_by_name(preludedb_sql_row_t *row, const char *column_name,
+                                        preludedb_sql_field_t **field)
+{
+        int column_num;
 
-        ret = preludedb_sql_field_new(field, row, column_num, value, len);
-        if ( ret < 0 )
-                return ret;
+        column_num = preludedb_sql_table_get_column_num(row->table, column_name);
+        if ( column_num < 0 )
+                return column_num;
 
-        return 1;
+        return preludedb_sql_row_get_field(row, column_num, field);
 }
 
 
@@ -894,6 +1182,7 @@ int preludedb_sql_row_fetch_field(preludedb_sql_row_t *row, unsigned int column_
  * @column_name: The column name of the field to be fetched.
  * @field: Pointer to the field object where the result will be stored.
  *
+ * DEPRECATED: use preludedb_sql_row_get_field_by_name() instead.
  * Fetch the field of column @column_name
  *
  * Returns: 1 if the row returns a non-empty field, 0 if it returns an empty field, or
@@ -902,17 +1191,10 @@ int preludedb_sql_row_fetch_field(preludedb_sql_row_t *row, unsigned int column_
 int preludedb_sql_row_fetch_field_by_name(preludedb_sql_row_t *row, const char *column_name,
                                           preludedb_sql_field_t **field)
 {
-        int column_num;
-
-        column_num = preludedb_sql_table_get_column_num(row->table, column_name);
-        if ( column_num < 0 )
-                return column_num;
-
-        return preludedb_sql_row_fetch_field(row, column_num, field);
+        return preludedb_sql_row_get_field_by_name(row, column_name, field);
 }
 
 
-
 /**
  * preludedb_sql_field_get_value:
  * @field: Pointer to a field object.
@@ -921,7 +1203,7 @@ int preludedb_sql_row_fetch_field_by_name(preludedb_sql_row_t *row, const char *
  *
  * Returns: field value.
  */
-const char *preludedb_sql_field_get_value(preludedb_sql_field_t *field)
+char *preludedb_sql_field_get_value(preludedb_sql_field_t *field)
 {
         return field->value;
 }
@@ -952,47 +1234,62 @@ size_t preludedb_sql_field_get_len(preludedb_sql_field_t *field)
  *
  * Returns: 0 on success, -1 if the wanted type does not match the field type.
  */
-#define preludedb_sql_field_to(name, type_name, format)                                         \
-int preludedb_sql_field_to_ ## name(preludedb_sql_field_t *field, type_name *value)             \
+#define preludedb_sql_field_to(name, rtype, func, min, max)                                     \
+int preludedb_sql_field_to_ ## name(preludedb_sql_field_t *field, name ## _t *value)            \
 {                                                                                               \
-        int ret;                                                                                \
+        rtype tmp;                                                                              \
+        char *eptr = NULL;                                                                      \
                                                                                                 \
-        ret = sscanf(preludedb_sql_field_get_value(field), format, value);                      \
-        if ( ret < 0 )                                                                          \
+        if ( min >= 0 && *field->value == '-' )                                                 \
                 return preludedb_error(PRELUDEDB_ERROR_INVALID_VALUE);                          \
                                                                                                 \
+        errno = 0;                                                                              \
+                                                                                                \
+        tmp = func(field->value, &eptr, 10);                                                    \
+        if ( tmp < min || tmp > max || (eptr && *eptr) || errno == ERANGE )                     \
+                return preludedb_error(PRELUDEDB_ERROR_INVALID_VALUE);                          \
+                                                                                                \
+        *value = (name ## _t) tmp;                                                              \
         return 0;                                                                               \
 }
 
-#define preludedb_sql_field_to_int8(name, type_name, format, min, max)                          \
-int preludedb_sql_field_to_ ## name(preludedb_sql_field_t *field, type_name *value)             \
-{                                                                                               \
-        int tmp, ret;                                                                           \
-                                                                                                \
-        ret = sscanf(preludedb_sql_field_get_value(field), format, &tmp);                       \
-        if ( ret <= 0 || tmp < min || tmp > max )                                               \
-                return preludedb_error(PRELUDEDB_ERROR_INVALID_VALUE);                          \
-                                                                                                \
-        *value = (type_name) tmp;                                                               \
-                                                                                                \
-        return 0;                                                                               \
+preludedb_sql_field_to(int8, long, strtol, PRELUDE_INT8_MIN, PRELUDE_INT8_MAX)
+preludedb_sql_field_to(uint8, unsigned long, strtoul, 0, PRELUDE_UINT8_MAX)
+preludedb_sql_field_to(int16, long, strtol, PRELUDE_INT16_MIN, PRELUDE_INT16_MAX)
+preludedb_sql_field_to(uint16, unsigned long, strtoul, 0, PRELUDE_UINT16_MAX)
+preludedb_sql_field_to(int32, long, strtol, PRELUDE_INT32_MIN, PRELUDE_INT32_MAX)
+preludedb_sql_field_to(uint32, unsigned long, strtoul, 0, PRELUDE_UINT32_MAX)
+preludedb_sql_field_to(int64, long long, strtoll, PRELUDE_INT64_MIN, PRELUDE_INT64_MAX)
+preludedb_sql_field_to(uint64, unsigned long long, strtoull, 0, PRELUDE_UINT64_MAX)
+
+int preludedb_sql_field_to_float(preludedb_sql_field_t *field, float *value)
+{
+        char *eptr = NULL;
+
+        errno = 0;
+
+        *value = strtof(preludedb_sql_field_get_value(field), &eptr);
+        if ( (eptr && *eptr) || errno == ERANGE )
+                return preludedb_error(PRELUDEDB_ERROR_INVALID_VALUE);
+
+        return 0;
 }
 
-/*
- * %hh is not a portable convertion specifier
- */
-preludedb_sql_field_to_int8(int8, int8_t, "%d", PRELUDE_INT8_MIN, PRELUDE_INT8_MAX)
-preludedb_sql_field_to_int8(uint8, uint8_t, "%u", 0, PRELUDE_UINT8_MAX)
-preludedb_sql_field_to(int16, int16_t, "%hd")
-preludedb_sql_field_to(uint16, uint16_t, "%hu")
-preludedb_sql_field_to(int32, int32_t, "%d")
-preludedb_sql_field_to(uint32, uint32_t, "%u")
-preludedb_sql_field_to(int64, int64_t, "%" PRELUDE_PRId64)
-preludedb_sql_field_to(uint64, uint64_t, "%" PRELUDE_PRIu64)
-preludedb_sql_field_to(float, float, "%f")
-preludedb_sql_field_to(double, double, "%lf")
 
 
+int preludedb_sql_field_to_double(preludedb_sql_field_t *field, double *value)
+{
+        char *eptr = NULL;
+
+        errno = 0;
+
+        *value = strtod(preludedb_sql_field_get_value(field), &eptr);
+        if ( (eptr && *eptr) || errno == ERANGE )
+                return preludedb_error(PRELUDEDB_ERROR_INVALID_VALUE);
+
+        return 0;
+}
+
 
 /**
  * preludedb_sql_field_to_string:
@@ -1169,7 +1466,7 @@ static int build_criterion_fixed_value(preludedb_sql_t *sql,
         if ( ret < 0 )
                 goto err;
 
-        ret = _preludedb_plugin_sql_build_constraint_string(sql->plugin, output, field, operator, prelude_string_get_string(value_str));
+        ret = _preludedb_plugin_sql_build_constraint_string(sql->plugin, sql->session, output, field, operator, prelude_string_get_string(value_str));
 
  err:
         prelude_string_destroy(value_str);
@@ -1217,7 +1514,7 @@ static int build_criterion_broken_down_time_equal(preludedb_sql_t *sql, prelude_
                                 return ret;
                 }
 
-                ret = _preludedb_plugin_sql_build_time_constraint_string(sql->plugin, output, field, tbl[i].type,
+                ret = _preludedb_plugin_sql_build_time_constraint_string(sql->plugin, sql->session, output, field, tbl[i].type,
                                                                          op, *tbl[i].field_ptr, 0);
                 if ( ret < 0 )
                         return ret;
@@ -1271,7 +1568,7 @@ static int build_criterion_regex(preludedb_sql_t *sql, prelude_string_t *output,
         if ( ret < 0 )
                 return ret;
 
-        ret = _preludedb_plugin_sql_build_constraint_string(sql->plugin, output, field, op, escaped);
+        ret = _preludedb_plugin_sql_build_constraint_string(sql->plugin, sql->session, output, field, op, escaped);
         free(escaped);
 
         return ret;
@@ -1279,6 +1576,30 @@ static int build_criterion_regex(preludedb_sql_t *sql, prelude_string_t *output,
 
 
 
+int preludedb_sql_build_time_extract_string(preludedb_sql_t *sql, prelude_string_t *output, const char *field,
+                                            preludedb_sql_time_constraint_type_t type, int gmt_offset)
+{
+        return _preludedb_plugin_sql_build_time_extract_string(sql->plugin, sql->session, output, field, type, gmt_offset);
+}
+
+
+
+int preludedb_sql_build_time_interval_string(preludedb_sql_t *sql, prelude_string_t *output, const char *field,
+                                             const char *value, preludedb_selected_object_interval_t unit)
+{
+        return _preludedb_plugin_sql_build_time_interval_string(sql->plugin, sql->session, output, field, value, unit);
+}
+
+
+
+int preludedb_sql_build_time_timezone_string(preludedb_sql_t *sql, prelude_string_t *output, const char *field, const char *tzvalue)
+{
+        return _preludedb_plugin_sql_build_time_timezone_string(sql->plugin, sql->session, output, field, tzvalue);
+}
+
+
+
+
 /**
  * preludedb_sql_build_criterion_string:
  * @sql: Pointer to a sql object.
@@ -1297,7 +1618,6 @@ int preludedb_sql_build_criterion_string(preludedb_sql_t *sql,
                                          idmef_criterion_operator_t operator, idmef_criterion_value_t *value)
 {
         int ret = -1;
-        const void *vptr;
         idmef_criterion_value_type_t type;
 
         if ( operator == IDMEF_CRITERION_OPERATOR_NULL )
@@ -1316,7 +1636,6 @@ int preludedb_sql_build_criterion_string(preludedb_sql_t *sql,
         }
 
         type = idmef_criterion_value_get_type(value);
-        vptr = idmef_criterion_value_get_value(value);
 
         if ( type == IDMEF_CRITERION_VALUE_TYPE_VALUE )
                 ret = build_criterion_fixed_value(sql, output, field, operator, idmef_criterion_value_get_value(value));
@@ -1415,7 +1734,7 @@ int preludedb_sql_time_to_timestamp(preludedb_sql_t *sql,
         if ( ! gmtime_r(&t, &utc) )
                 return prelude_error_from_errno(errno);
 
-        ret = _preludedb_plugin_sql_build_timestamp_string(sql->plugin, &utc, time_buf, time_buf_size);
+        ret = _preludedb_plugin_sql_build_timestamp_string(sql->plugin, sql->session, &utc, time_buf, time_buf_size);
         if ( ret < 0 )
                 return ret;
 
@@ -1430,6 +1749,57 @@ int preludedb_sql_time_to_timestamp(preludedb_sql_t *sql,
 
 
 
+/**
+ * preludedb_sql_get_server_version:
+ * @sql: Pointer to a sql object.
+ *
+ * Get sql server version.
+ *
+ * Returns: A negative value if an error occur, the version number otherwise.
+ */
+long preludedb_sql_get_server_version(preludedb_sql_t *sql)
+{
+        gl_recursive_lock_lock(sql->mutex);
+
+        assert_connected(sql);
+
+        gl_recursive_lock_unlock(sql->mutex);
+
+        return _preludedb_plugin_sql_get_server_version(sql->plugin, sql->session);
+}
+
+
+
+/**
+ * preludedb_sql_get_type:
+ * @sql: Pointer to a sql object.
+ *
+ * Get the type of the SQL database (specified by the user at
+ * @sql initialization time).
+ *
+ * Returns: the type of the SQL database.
+ */
+const char *preludedb_sql_get_type(const preludedb_sql_t *sql)
+{
+        return sql->type;
+}
+
+
+
+/**
+ * preludedb_sql_get_settings:
+ * @sql: Pointer to a sql object.
+ *
+ * Returns: The #preludedb_sql_settings_t object associated with @sql.
+ */
+const preludedb_sql_settings_t *preludedb_sql_get_settings(const preludedb_sql_t *sql)
+{
+        return sql->settings;
+}
+
+
+
+
 /**
  * preludedb_sql_get_plugin_error:
  * @sql: Pointer to a sql object.
