@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2017 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the PreludeDB library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -28,22 +28,25 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <libprelude/prelude.h>
+#include <libprelude/idmef-criteria.h>
 
 
 #include "preludedb-error.h"
 #include "preludedb-sql-settings.h"
-#include "preludedb-sql.h"
 #include "preludedb-path-selection.h"
 
 #include "preludedb.h"
+#include "preludedb-sql.h"
 #include "preludedb-plugin-format.h"
 #include "preludedb-plugin-format-prv.h"
 
 
 #define PRELUDEDB_PLUGIN_SYMBOL "preludedb_plugin_init"
+#define PRELUDEDB_ENOTSUP(x) preludedb_error_verbose(prelude_error_code_from_errno(ENOSYS), "Database format does not support '%s' operation", x)
 
 
 struct preludedb {
+        int refcount;
         char *format_version;
         preludedb_sql_t *sql;
         preludedb_plugin_format_t *plugin;
@@ -52,17 +55,17 @@ struct preludedb {
 struct preludedb_result_idents {
         preludedb_t *db;
         void *res;
+        int refcount;
 };
 
 struct preludedb_result_values {
+        int refcount;
         preludedb_t *db;
         preludedb_path_selection_t *selection;
         void *res;
 };
 
-
-
-
+preludedb_plugin_format_t *_preludedb_get_plugin_format(preludedb_t *db);
 int _preludedb_sql_transaction_start(preludedb_sql_t *sql);
 int _preludedb_sql_transaction_end(preludedb_sql_t *sql);
 int _preludedb_sql_transaction_abort(preludedb_sql_t *sql);
@@ -154,7 +157,7 @@ static int preludedb_autodetect_format(preludedb_t *db)
         if ( ret < 0 )
                 goto error;
 
-        ret = preludedb_sql_row_fetch_field(row, 0, &format_name);
+        ret = preludedb_sql_row_get_field(row, 0, &format_name);
         if ( ret < 0 )
                 goto error;
 
@@ -162,7 +165,7 @@ static int preludedb_autodetect_format(preludedb_t *db)
         if ( ret < 0 )
                 goto error;
 
-        ret = preludedb_sql_row_fetch_field(row, 1, &format_version);
+        ret = preludedb_sql_row_get_field(row, 1, &format_version);
         if ( ret < 0 )
                 goto error;
 
@@ -200,6 +203,11 @@ int preludedb_new(preludedb_t **db, preludedb_sql_t *sql, const char *format_nam
 {
         int ret;
 
+        /*
+         * FIXME: format_name, errbuf, are deprecated.
+         */
+        prelude_return_val_if_fail(sql, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         *db = calloc(1, sizeof (**db));
         if ( ! *db ) {
                 ret = preludedb_error_from_errno(errno);
@@ -207,13 +215,17 @@ int preludedb_new(preludedb_t **db, preludedb_sql_t *sql, const char *format_nam
                 return ret;
         }
 
-        (*db)->sql = sql;
+        (*db)->refcount = 1;
+        (*db)->sql = preludedb_sql_ref(sql);
 
         if ( format_name )
                 ret = preludedb_set_format(*db, format_name);
         else
                 ret = preludedb_autodetect_format(*db);
 
+        if ( ret >= 0 && (*db)->plugin->init )
+                ret = (*db)->plugin->init(*db);
+
         if ( ret < 0 ) {
                 if ( errbuf )
                         preludedb_get_error(*db, ret, errbuf, size);
@@ -229,6 +241,16 @@ int preludedb_new(preludedb_t **db, preludedb_sql_t *sql, const char *format_nam
 
 
 
+
+preludedb_t *preludedb_ref(preludedb_t *db)
+{
+        db->refcount++;
+        return db;
+}
+
+
+
+
 /**
  * preludedb_destroy:
  * @db: Pointer to a db object.
@@ -237,6 +259,9 @@ int preludedb_new(preludedb_t **db, preludedb_sql_t *sql, const char *format_nam
  */
 void preludedb_destroy(preludedb_t *db)
 {
+        if ( --db->refcount != 0 )
+                return;
+
         preludedb_sql_destroy(db->sql);
         free(db->format_version);
         free(db);
@@ -252,6 +277,7 @@ void preludedb_destroy(preludedb_t *db)
  */
 const char *preludedb_get_format_name(preludedb_t *db)
 {
+        prelude_return_val_if_fail(db, NULL);
         return prelude_plugin_get_name(db->plugin);
 }
 
@@ -265,6 +291,7 @@ const char *preludedb_get_format_name(preludedb_t *db)
  */
 const char *preludedb_get_format_version(preludedb_t *db)
 {
+        prelude_return_val_if_fail(db, NULL);
         return db->format_version;
 }
 
@@ -281,6 +308,8 @@ const char *preludedb_get_format_version(preludedb_t *db)
  */
 int preludedb_set_format(preludedb_t *db, const char *format_name)
 {
+        prelude_return_val_if_fail(db && format_name, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         db->plugin = (preludedb_plugin_format_t *) prelude_plugin_search_by_name(&plugin_format_list, format_name);
         if ( ! db->plugin )
                 return preludedb_error_verbose(PRELUDEDB_ERROR_CANNOT_LOAD_FORMAT_PLUGIN,
@@ -299,6 +328,7 @@ int preludedb_set_format(preludedb_t *db, const char *format_name)
  */
 preludedb_sql_t *preludedb_get_sql(preludedb_t *db)
 {
+        prelude_return_val_if_fail(db, NULL);
         return db->sql;
 }
 
@@ -314,6 +344,8 @@ preludedb_sql_t *preludedb_get_sql(preludedb_t *db)
  * Build an error message from the error code given as argument and from
  * the sql plugin error string (if any) if the error code is db related.
  *
+ * FIXME: deprecated.
+ *
  * Returns: a pointer to the error string or NULL if an error occured.
  */
 char *preludedb_get_error(preludedb_t *db, preludedb_error_t error, char *errbuf, size_t size)
@@ -343,11 +375,22 @@ char *preludedb_get_error(preludedb_t *db, preludedb_error_t error, char *errbuf
  */
 int preludedb_insert_message(preludedb_t *db, idmef_message_t *message)
 {
-        return db->plugin->insert_message(db->sql, message);
+        prelude_return_val_if_fail(db && message, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        return db->plugin->insert_message(db, message);
 }
 
 
 
+preludedb_result_idents_t *preludedb_result_idents_ref(preludedb_result_idents_t *results)
+{
+        prelude_return_val_if_fail(results, NULL);
+
+        results->refcount++;
+        return results;
+}
+
+
 /**
  * preludedb_result_idents_destroy:
  * @result: Pointer to an idents result object.
@@ -356,27 +399,53 @@ int preludedb_insert_message(preludedb_t *db, idmef_message_t *message)
  */
 void preludedb_result_idents_destroy(preludedb_result_idents_t *result)
 {
+        prelude_return_if_fail(result);
+
+        if ( --result->refcount != 0 )
+                return;
+
         result->db->plugin->destroy_message_idents_resource(result->res);
+        preludedb_destroy(result->db);
+
         free(result);
 }
 
 
+
 /**
- * preludedb_result_idents_get_next:
+ * preludedb_result_idents_get:
  * @result: Pointer to an idents result object.
+ * @row_index: Row index to retrieve the ident from.
  * @ident: Pointer to an ident where the next ident will be stored.
  *
- * Retrieve the next ident from the idents result object.
+ * Retrieve the ident located at @row_index from the idents result object.
  *
  * Returns: 1 if an ident is available, 0 if there is no more idents available or
  * a negative value if an error occur.
  */
-int preludedb_result_idents_get_next(preludedb_result_idents_t *result, uint64_t *ident)
+int preludedb_result_idents_get(preludedb_result_idents_t *result, unsigned int row_index, uint64_t *ident)
 {
-        return result->db->plugin->get_next_message_ident(result->res, ident);
+        prelude_return_val_if_fail(result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! result->db->plugin->get_message_ident )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement ident retrieval by index");
+
+        return result->db->plugin->get_message_ident(result->res, row_index, ident);
 }
 
 
+unsigned int preludedb_result_idents_get_count(preludedb_result_idents_t *result)
+{
+        prelude_return_val_if_fail(result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! result->db->plugin->get_message_ident_count )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement ident count retrieval");
+
+        return result->db->plugin->get_message_ident_count(result->res);
+}
+
+
+
 /**
  * preludedb_result_values_destroy:
  * @result: Pointer to a result values object.
@@ -385,33 +454,91 @@ int preludedb_result_idents_get_next(preludedb_result_idents_t *result, uint64_t
  */
 void preludedb_result_values_destroy(preludedb_result_values_t *result)
 {
+        prelude_return_if_fail(result);
+
+        if ( --result->refcount != 0 )
+                return;
+
         result->db->plugin->destroy_values_resource(result->res);
+        preludedb_path_selection_destroy(result->selection);
+        preludedb_destroy(result->db);
+
         free(result);
 }
 
 
 
-/**
- * preludedb_result_values_get_next:
- * @result: Pointer to a values result object.
- * @values: Pointer to a values array where the next row of values will be stored.
- *
- * Retrieve the next values row from the values result object.
- *
- * Returns: the number of returned values, 0 if there are no values or a negative value if an
- * error occur.
- */
-int preludedb_result_values_get_next(preludedb_result_values_t *result, idmef_value_t ***values)
+preludedb_path_selection_t *preludedb_result_values_get_selection(preludedb_result_values_t *result)
 {
-        return result->db->plugin->get_next_values(result->res, result->selection, values);
+        prelude_return_val_if_fail(result, NULL);
+        return result->selection;
 }
 
 
+int preludedb_result_values_get_row(preludedb_result_values_t *result, unsigned int rownum, void **row)
+{
+        prelude_return_val_if_fail(result && row, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! result->db->plugin->get_result_values_row )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement value selection");
+
+        return result->db->plugin->get_result_values_row(result, rownum, row);
+}
+
+
+
+static int internal_value_cb(void **out, void *data, size_t size, idmef_value_type_id_t type)
+{
+        if ( ! type && ! data ) {
+                *out = NULL;
+                return 0;
+        }
+
+        else if ( type == IDMEF_VALUE_TYPE_STRING ) {
+                int ret;
+                prelude_string_t *str;
+
+                ret = prelude_string_new_dup_fast(&str, data, size);
+                if ( ret < 0 )
+                        return ret;
+
+                return idmef_value_new_string((idmef_value_t **) out, str);
+        }
+
+        else if ( type == IDMEF_VALUE_TYPE_TIME )
+                return idmef_value_new_time((idmef_value_t **) out, idmef_time_ref(data));
+
+        return idmef_value_new_from_string((idmef_value_t **) out, type, data);
+}
+
+
+int preludedb_result_values_get_field(preludedb_result_values_t *result, void *row, preludedb_selected_path_t *selected, idmef_value_t **field)
+{
+        prelude_return_val_if_fail(result && row && field, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! result->db->plugin->get_result_values_field )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement value selection");
+
+        return result->db->plugin->get_result_values_field(result, row, selected, internal_value_cb, (void **) field);
+}
+
+
+int preludedb_result_values_get_field_direct(preludedb_result_values_t *result, void *row, preludedb_selected_path_t *selected,
+                                             preludedb_result_values_get_field_cb_func_t cb, void **out)
+{
+        prelude_return_val_if_fail(result && row && cb && out, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! result->db->plugin->get_result_values_field )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement value selection");
+
+        return result->db->plugin->get_result_values_field(result, row, selected, cb, out);
+}
+
 
 static int
 preludedb_get_message_idents(preludedb_t *db,
                              idmef_criteria_t *criteria,
-                             int (*get_idents)(preludedb_sql_t *sql, idmef_criteria_t *criteria,
+                             int (*get_idents)(preludedb_t *db, idmef_criteria_t *criteria,
                                                int limit, int offset,
                                                preludedb_result_idents_order_t order,
                                                void **res),
@@ -425,11 +552,14 @@ preludedb_get_message_idents(preludedb_t *db,
         if ( ! *result )
                 return preludedb_error_from_errno(errno);
 
-        (*result)->db = db;
-
-        ret = get_idents(db->sql, criteria, limit, offset, order, &(*result)->res);
-        if ( ret <= 0 )
+        ret = get_idents(db, criteria, limit, offset, order, &(*result)->res);
+        if ( ret <= 0 ) {
                 free(*result);
+                return ret;
+        }
+
+        (*result)->refcount++;
+        (*result)->db = preludedb_ref(db);
 
         return ret;
 }
@@ -452,6 +582,7 @@ int preludedb_get_alert_idents(preludedb_t *db,
                                preludedb_result_idents_order_t order,
                                preludedb_result_idents_t **result)
 {
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
         return preludedb_get_message_idents(db, criteria, db->plugin->get_alert_idents, limit, offset, order, result);
 }
 
@@ -473,6 +604,7 @@ int preludedb_get_heartbeat_idents(preludedb_t *db,
                                    preludedb_result_idents_order_t order,
                                    preludedb_result_idents_t **result)
 {
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
         return preludedb_get_message_idents(db, criteria, db->plugin->get_heartbeat_idents, limit, offset, order, result);
 }
 
@@ -488,7 +620,8 @@ int preludedb_get_heartbeat_idents(preludedb_t *db,
  */
 int preludedb_get_alert(preludedb_t *db, uint64_t ident, idmef_message_t **message)
 {
-        return db->plugin->get_alert(db->sql, ident, message);
+        prelude_return_val_if_fail(db && message, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return db->plugin->get_alert(db, ident, message);
 }
 
 
@@ -503,7 +636,8 @@ int preludedb_get_alert(preludedb_t *db, uint64_t ident, idmef_message_t **messa
  */
 int preludedb_get_heartbeat(preludedb_t *db, uint64_t ident, idmef_message_t **message)
 {
-        return db->plugin->get_heartbeat(db->sql, ident, message);
+        prelude_return_val_if_fail(db && message, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return db->plugin->get_heartbeat(db, ident, message);
 }
 
 
@@ -518,7 +652,8 @@ int preludedb_get_heartbeat(preludedb_t *db, uint64_t ident, idmef_message_t **m
  */
 int preludedb_delete_alert(preludedb_t *db, uint64_t ident)
 {
-        return db->plugin->delete_alert(db->sql, ident);
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return db->plugin->delete_alert(db, ident);
 }
 
 
@@ -532,12 +667,14 @@ int preludedb_delete_alert(preludedb_t *db, uint64_t ident)
  *
  * Returns: the number of alert deleted on success, or a negative value if an error occur.
  */
-ssize_t preludedb_delete_alert_from_list(preludedb_t *db, uint64_t *ident, size_t size)
+ssize_t preludedb_delete_alert_from_list(preludedb_t *db, uint64_t *idents, size_t isize)
 {
-        if ( size == 0 )
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( isize == 0 )
                 return 0;
 
-        return _preludedb_plugin_format_delete_alert_from_list(db->plugin, db->sql, ident, size);
+        return _preludedb_plugin_format_delete_alert_from_list(db->plugin, db, idents, isize);
 }
 
 
@@ -552,7 +689,8 @@ ssize_t preludedb_delete_alert_from_list(preludedb_t *db, uint64_t *ident, size_
  */
 ssize_t preludedb_delete_alert_from_result_idents(preludedb_t *db, preludedb_result_idents_t *result)
 {
-        return _preludedb_plugin_format_delete_alert_from_result_idents(db->plugin, db->sql, result);
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return _preludedb_plugin_format_delete_alert_from_result_idents(db->plugin, db, result);
 }
 
 
@@ -568,7 +706,8 @@ ssize_t preludedb_delete_alert_from_result_idents(preludedb_t *db, preludedb_res
  */
 int preludedb_delete_heartbeat(preludedb_t *db, uint64_t ident)
 {
-        return db->plugin->delete_heartbeat(db->sql, ident);
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return db->plugin->delete_heartbeat(db, ident);
 }
 
 
@@ -582,12 +721,14 @@ int preludedb_delete_heartbeat(preludedb_t *db, uint64_t ident)
  *
  * Returns: the number of heartbeat deleted on success, or a negative value if an error occur.
  */
-ssize_t preludedb_delete_heartbeat_from_list(preludedb_t *db, uint64_t *idents, size_t size)
+ssize_t preludedb_delete_heartbeat_from_list(preludedb_t *db, uint64_t *idents, size_t isize)
 {
-        if ( size == 0 )
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( isize == 0 )
                 return 0;
 
-        return _preludedb_plugin_format_delete_heartbeat_from_list(db->plugin, db->sql, idents, size);
+        return _preludedb_plugin_format_delete_heartbeat_from_list(db->plugin, db, idents, isize);
 }
 
 
@@ -603,7 +744,53 @@ ssize_t preludedb_delete_heartbeat_from_list(preludedb_t *db, uint64_t *idents,
  */
 ssize_t preludedb_delete_heartbeat_from_result_idents(preludedb_t *db, preludedb_result_idents_t *result)
 {
-        return _preludedb_plugin_format_delete_heartbeat_from_result_idents(db->plugin, db->sql, result);
+        prelude_return_val_if_fail(db && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return _preludedb_plugin_format_delete_heartbeat_from_result_idents(db->plugin, db, result);
+}
+
+
+
+/**
+ * preludedb_delete:
+ * @db: Pointer to a db object.
+ * @criteria: Pointer to a criteria object.
+ *
+ * Delete all database object matching @criteria.
+ *
+ * Returns: a negative value if an error occured.
+ */
+int preludedb_delete(preludedb_t *db, idmef_criteria_t *criteria)
+{
+        int ret;
+        preludedb_result_idents_t *result;
+
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( db->plugin->delete )
+                return db->plugin->delete(db, criteria);
+
+        ret = idmef_criteria_get_class(criteria);
+        if ( ret < 0 )
+                return ret;
+
+        if ( ret == IDMEF_CLASS_ID_ALERT ) {
+                ret = preludedb_get_alert_idents(db, criteria, -1, 0, 0, &result);
+                if ( ret <= 0 )
+                        return ret;
+
+                ret = preludedb_delete_alert_from_result_idents(db, result);
+        }
+
+        else if ( IDMEF_CLASS_ID_HEARTBEAT ) {
+                ret = preludedb_get_heartbeat_idents(db, criteria, -1, 0, 0, &result);
+                if ( ret <= 0 )
+                        return ret;
+
+                ret = preludedb_delete_heartbeat_from_result_idents(db, result);
+        }
+
+        preludedb_result_idents_destroy(result);
+        return ret;
 }
 
 
@@ -618,7 +805,7 @@ ssize_t preludedb_delete_heartbeat_from_result_idents(preludedb_t *db, preludedb
  * @offset: Offset in results or -1 if no offset.
  * @result: Values result.
  *
- * Returns: the number of result or a negative value if an error occured.
+ * Returns: 1 if there are result, 0 if there are none, or a negative value if an error occured.
  */
 int preludedb_get_values(preludedb_t *db,
                          preludedb_path_selection_t *path_selection,
@@ -629,22 +816,150 @@ int preludedb_get_values(preludedb_t *db,
 {
         int ret;
 
+        prelude_return_val_if_fail(db && path_selection && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         *result = calloc(1, sizeof (**result));
         if ( ! *result )
                 return preludedb_error_from_errno(errno);
 
-        (*result)->db = db;
-        (*result)->selection = path_selection;
-
-        ret = db->plugin->get_values(db->sql, path_selection, criteria, distinct, limit, offset, &(*result)->res);
-        if ( ret <= 0 )
+        ret = db->plugin->get_values(db , path_selection, criteria, distinct, limit, offset, &(*result)->res);
+        if ( ret <= 0 ) {
                 free(*result);
+                *result = NULL;
+                return ret;
+        }
+
+        (*result)->refcount = 1;
+        (*result)->db = preludedb_ref(db);
+        (*result)->selection = preludedb_path_selection_ref(path_selection);
 
         return ret;
 }
 
 
 
+void *preludedb_result_values_get_data(preludedb_result_values_t *results)
+{
+        prelude_return_val_if_fail(results, NULL);
+        return results->res;
+}
+
+
+
+preludedb_t *preludedb_result_values_get_db(preludedb_result_values_t *results)
+{
+        prelude_return_val_if_fail(results, NULL);
+        return results->db;
+}
+
+
+int preludedb_result_values_get_count(preludedb_result_values_t *results)
+{
+        prelude_return_val_if_fail(results, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! results->db->plugin->get_result_values_count )
+                return preludedb_error_verbose(PRELUDEDB_ERROR_GENERIC, "format plugin doesn't implement value count retrieval");
+
+        return results->db->plugin->get_result_values_count(results);
+}
+
+
+
+preludedb_result_values_t *preludedb_result_values_ref(preludedb_result_values_t *results)
+{
+        prelude_return_val_if_fail(results, NULL);
+
+        results->refcount++;
+        return results;
+}
+
+
+unsigned int preludedb_result_values_get_field_count(preludedb_result_values_t *results)
+{
+        prelude_return_val_if_fail(results, prelude_error(PRELUDE_ERROR_ASSERTION));
+        return preludedb_path_selection_get_count(results->selection);
+}
+
+
+ssize_t preludedb_update_from_list(preludedb_t *db,
+                                   const idmef_path_t * const *paths, const idmef_value_t * const *values, size_t pvsize,
+                                   uint64_t *idents, size_t isize)
+{
+        prelude_return_val_if_fail(db && paths && values, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! db->plugin->update_from_list )
+                return preludedb_error_from_errno(ENOSYS);
+
+        return db->plugin->update_from_list(db, paths, values, pvsize, idents, isize);
+}
+
+
+ssize_t preludedb_update_from_result_idents(preludedb_t *db,
+                                            const idmef_path_t * const *paths, const idmef_value_t * const *values, size_t pvsize,
+                                            preludedb_result_idents_t *result)
+{
+        prelude_return_val_if_fail(db && paths && values && result, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! db->plugin->update_from_result_idents )
+                return PRELUDEDB_ENOTSUP("update_from_result_ident");
+
+        return db->plugin->update_from_result_idents(db, paths, values, pvsize, result);
+}
+
+
+
+/**
+ * preludedb_update:
+ * @db: Pointer to a db object.
+ * @paths: Array of path to update
+ * @values: Array of value for their respective @paths.
+ * @pvsize: Number of element in the @paths and @values array.
+ * @criteria: Criteria updated event should match.
+ * @order: Optional list of path used to order the update command.
+ * @limit: Limit of results or -1 if no limit.
+ * @offset: Offset in results or -1 if no offset.
+ *
+ *
+ * Returns: the number of result or a negative value if an error occured.
+ */
+
+int preludedb_update(preludedb_t *db,
+                     const idmef_path_t * const *paths, const idmef_value_t * const *values, size_t pvsize,
+                     idmef_criteria_t *criteria,
+                     preludedb_path_selection_t *order,
+                     int limit, int offset)
+{
+        prelude_return_val_if_fail(db && paths && values, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! db->plugin->update )
+                return PRELUDEDB_ENOTSUP("update");
+
+        return db->plugin->update(db, paths, values, pvsize, criteria, order, limit, offset);
+}
+
+
+
+/**
+ * preludedb_optimize:
+ * @db: Pointer to a db object.
+ *
+ * Call a database optimization function, if supported.
+ *
+ * Returns: 0 on success or a negative value if an error occurred.
+ */
+int preludedb_optimize(preludedb_t *db)
+{
+        prelude_return_val_if_fail(db, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( ! db->plugin->optimize )
+                return PRELUDEDB_ENOTSUP("optimize");
+
+        return db->plugin->optimize(db);
+}
+
+
+
+
 /**
  * preludedb_transaction_start:
  * @db: Pointer to a #preludedb_t object.
@@ -659,6 +974,8 @@ int preludedb_transaction_start(preludedb_t *db)
 {
         int ret;
 
+        prelude_return_val_if_fail(db && db->sql, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         ret = _preludedb_sql_transaction_start(db->sql);
         if ( ret < 0 )
                 return ret;
@@ -684,6 +1001,8 @@ int preludedb_transaction_end(preludedb_t *db)
 {
         int ret;
 
+        prelude_return_val_if_fail(db && db->sql, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         ret = _preludedb_sql_transaction_end(db->sql);
         _preludedb_sql_enable_internal_transaction(db->sql);
 
@@ -709,6 +1028,8 @@ int preludedb_transaction_abort(preludedb_t *db)
 {
         int ret;
 
+        prelude_return_val_if_fail(db && db->sql, prelude_error(PRELUDE_ERROR_ASSERTION));
+
         ret = _preludedb_sql_transaction_abort(db->sql);
         _preludedb_sql_enable_internal_transaction(db->sql);
 
@@ -717,3 +1038,9 @@ int preludedb_transaction_abort(preludedb_t *db)
 
         return ret;
 }
+
+
+preludedb_plugin_format_t *_preludedb_get_plugin_format(preludedb_t *db)
+{
+        return db->plugin;
+}
